@@ -1,28 +1,41 @@
-/*	Copyright (C) 1993 Free Software Foundation, Inc.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this software; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+/*
+ * $Id: cmd.c,v 1.19 2000/11/22 19:33:00 danny Exp $
+ *
+ * Copyright  1993, 1999, 2000 Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this software; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include <ctype.h>
 #include <errno.h>
 #include <stdarg.h>
 #include "sysdef.h"
 #include <termios.h>
+#include <malloc.h>
+
+#ifdef	HAVE_MOTIF
+#include "io-motif.h"
+#endif
 
 #ifndef _DEBUG_MALLOC_INC
 #define local_free free
@@ -352,7 +365,7 @@ error_alarm ()
 	{
 	  pop_unfinished_command ();
 	  alarm_table[2].freq = 0;
-	  longjmp (error_exception, 1);
+	  longjmp (Global->error_exception, 1);
 	}
     }
   else
@@ -445,7 +458,7 @@ block_until_excitment (int timeout_seconds)
 int 
 real_get_chr (void)
 {
-  int ch = EOF;			/* The char that will be returned. */
+  unsigned int ch = EOF;		/* The char that will be returned. */
 
   /* Characters with the meta bit set are returned as
    * two characters: ESC and a non-meta character.
@@ -456,7 +469,7 @@ real_get_chr (void)
   static int saved_char;
 
   /* A buffer of characters read in one burst from the kbd. */
-  static char ibuf[256];
+  static unsigned char ibuf[256];
   static int i_in;		/* chars buffered */
   static int i_cnt;		/* buffer position */
 
@@ -484,9 +497,9 @@ real_get_chr (void)
       io_scan_for_input (0);
       if (!io_input_avail ())
 	{
-	  if (auto_recalc && eval_next_cell ())
+	  if (Global->auto_recalc && eval_next_cell ())
 	    {
-	      if (bkgrnd_recalc)
+	      if (Global->bkgrnd_recalc)
 		while (!io_input_avail () && eval_next_cell ())
 		  io_scan_for_input (0);
 	      else
@@ -500,10 +513,10 @@ real_get_chr (void)
 	    }
 	  else
 	    {
-	      int timeout = (alarm_active
-			     ? (alarm_seconds == 1
+	      int timeout = (Global->alarm_active
+			     ? (Global->alarm_seconds == 1
 				? 1
-				: (alarm_seconds / 2))
+				: (Global->alarm_seconds / 2))
 			     : 0);
 			     
 	      io_redisp ();
@@ -536,11 +549,25 @@ real_get_chr (void)
 
 fini:
 
-  if (ch & 0x80)
-    {
+  if (ch & 0x80) {
+#if 0
       saved_char = 1 + (ch & 0x7f);
       ch = CTRL ('[');
-    }
+#else	/* SCANDI */
+	switch (ch) {
+		case 229:                             /* e */
+		case 228:                             /* d */
+		case 246:                             /* v */
+		case 197:                             /* E */
+		case 196:                             /* D */
+		case 214:                             /* V */
+			break;
+		default:
+			saved_char = 1 + (ch & 0x7f);
+			ch = CTRL ('[');
+	}
+#endif
+  }
 
   if (making_macro)
     {
@@ -565,6 +592,19 @@ fini:
   return ch;
 }
 
+void OleoLog(char *fmt, ...)
+{
+#if 0
+	va_list	ap;
+	FILE	*log = fopen("/tmp/oleolog", "a");
+	if (log) {
+		va_start(ap, fmt);
+		vfprintf(log, fmt, ap);
+		va_end(ap);
+		fclose(log);
+	}
+#endif
+}
 
 /*****************************************************************
  * 
@@ -603,7 +643,8 @@ struct command_frame * the_cmd_frame = 0;
 struct command_frame * running_frames = 0;
 
 
-/* This is called when the current frame has keymapped
+/*
+ * This is called when the current frame has keymapped
  * down to some function (stored in the_cmd_frame->_cur_cmd.
  * This pushes a new frame in which the arguments to that
  * command will be stored.
@@ -614,7 +655,7 @@ struct command_frame * running_frames = 0;
  */
 
 void
-push_command_frame (struct rng * rng, char * first_line, int len)
+push_command_frame(struct rng * rng, char * first_line, int len)
 {
   struct command_frame * new_cf =
     (struct command_frame *)ck_malloc (sizeof (*new_cf));
@@ -695,6 +736,7 @@ push_command_frame (struct rng * rng, char * first_line, int len)
 	new_cf->_mkcol = mkcol;
 	new_cf->_setrow = setrow;
 	new_cf->_setcol = setcol;
+
 	if (!rng)
 	  new_cf->input = the_cmd_frame->input;
       }
@@ -912,7 +954,7 @@ exit_minibuffer (void)
 	  the_cmd_arg.is_set = 1;
 	  input_active = 0;
 	  window_after_input = -1;
-	  topclear = 2;
+	  Global->topclear = 2;
 	}
     }
 }
@@ -942,13 +984,16 @@ set_default_arg (struct command_arg * arg, char * text, int len)
   setn_arg_text (arg, text, len);
 }
 
-/* This is the main loop of oleo.   It reads commands and their arguments, and
- * evaluates them.  It (via real_get_chr) udpates the display and performs
- * background recomputation.
+/*
+ * This is the main loop of oleo.
+ *
+ * It reads commands and their arguments, and evaluates them.
+ * It (via real_get_chr) udpates the display and performs background recomputation.
  *
  * This function can also be used to evaluate a function without doing any
- * interaction.  This is done by pushing a macro_only command frame
- *  (see execute_command).
+ * interaction.
+ *
+ * This is done by pushing a macro_only command frame (see execute_command).
  */
 
 void
@@ -961,9 +1006,10 @@ command_loop (int prefix, int iscmd)
   if (the_cmd_frame->cmd)
     goto resume_getting_arguments;
 
-  /* Commands (notably execute_command) just tweek the command_frame
+  /*
+   * Commands (notably execute_command) just tweek the command_frame
    * state for some other command.  To accomplish this, there is an 
-   * entry point that avoid reinitializing the command_frame.\
+   * entry point that avoid reinitializing the command_frame.
    */
   if (prefix)
     {
@@ -1287,8 +1333,9 @@ command_loop (int prefix, int iscmd)
 	       */
 	      char * prompt = the_cmd_arg.arg_desc;
 	      
-	      switch (*prompt)
-		{
+	OleoLog("Prompt [%s]\n", prompt);
+
+	      switch (*prompt) {	/* Main prompt */
 		case 'c':
 		  {
                     int tmp;
@@ -1436,7 +1483,7 @@ command_loop (int prefix, int iscmd)
 		    goto next_arg;
 		  }
 		case 'M':
-		  if (modified)
+		  if (Global->modified)
 		    {
 		      ++prompt;
 		      if (get_argument (prompt, &yes_style))
@@ -1454,30 +1501,21 @@ command_loop (int prefix, int iscmd)
 		case 'p':
 		  {
 		    ++prompt;
-		    switch (*prompt)
-		      {
-		      default:
-			the_cmd_arg.val.integer
-			  = the_cmd_frame->prev->_how_many;
-			the_cmd_arg.is_set = 1;
-			the_cmd_arg.do_prompt = 0;
-			the_cmd_arg.style = &int_constant_style;
-			init_arg_text
-			  (&the_cmd_arg,
-			   long_to_str ((long)the_cmd_arg.val.integer)); 
-			break;
 
-		      case '?':	/* Command wants to know if prefix provided */
-			the_cmd_arg.val.integer =
-			  (the_cmd_frame->prev->_raw_prefix.alloc
-			   && the_cmd_frame->prev->_raw_prefix.buf[0]);
+		    if (*prompt == '?') {	/* Command wants to know if prefix provided */
+			the_cmd_arg.val.integer = (the_cmd_frame->prev->_raw_prefix.alloc
+				&& the_cmd_frame->prev->_raw_prefix.buf[0]);
 			the_cmd_arg.is_set = 1;
 			the_cmd_arg.do_prompt = 0;
 			the_cmd_arg.style = &int_constant_style;
-			init_arg_text (&the_cmd_arg,
-				       the_cmd_arg.val.integer ? "1" : "0");
-			break;
-		      }
+			init_arg_text(&the_cmd_arg, the_cmd_arg.val.integer ? "1" : "0");
+		    } else {
+			the_cmd_arg.val.integer = the_cmd_frame->prev->_how_many;
+			the_cmd_arg.is_set = 1;
+			the_cmd_arg.do_prompt = 0;
+			the_cmd_arg.style = &int_constant_style;
+			init_arg_text(&the_cmd_arg, long_to_str((long)the_cmd_arg.val.integer)); 
+		    }
 		    goto next_arg;
 		  }
 		case 'N':
@@ -1544,19 +1582,27 @@ command_loop (int prefix, int iscmd)
  * 
  * --FB, 1997.12.27
  */
+
+#define mark_is_set (mkrow != NON_ROW)
+#define interactive_mode (!rmac || iscmd)
+
                 case '@':
 		case 'r':
 		case 'R':
 		  {
-		    if (*prompt != '@' && mkrow != NON_ROW)
-                      {
+#if 0
+	/*
+	 * I don't know what I'm breaking by uncommenting this.
+	 * Danny 18/7/2000.
+	 */
+		    if (*prompt != '@' && !mark_is_set) {
+			/* Default to current cell */
                         mkrow = curow;
                         mkcol = cucol;
-                      }
-                    if ((*prompt != 'R'
-                        && (!rmac || iscmd)
-                        && mkrow != NON_ROW) || *prompt =='@')
-		      {
+                    }
+#endif
+                    if ((*prompt != 'R' && interactive_mode
+				&& mark_is_set) || *prompt =='@') {
 			the_cmd_arg.val.range.lr = MIN(mkrow, curow);
 			the_cmd_arg.val.range.hr = MAX(mkrow, curow);
 			the_cmd_arg.val.range.lc = MIN(mkcol, cucol);
@@ -1564,8 +1610,7 @@ command_loop (int prefix, int iscmd)
                           mkrow = NON_ROW;
                           mkcol = NON_COL;
                           io_update_status ();
-                        if (*prompt == '@' && (!rmac || iscmd))
-                          {
+                        if (*prompt == '@' && interactive_mode) {
                             ++prompt;
                             if (get_argument (prompt, &range_style))
                               {
@@ -1573,20 +1618,16 @@ command_loop (int prefix, int iscmd)
                                   range_name (&the_cmd_arg.val.range));
                               }
                             goto new_cycle;
-                          }
-                        else
-                          {
+                        } else {	/* (Noninteractive mode and @) or r */
                             ++prompt;
                             the_cmd_arg.is_set = 1;
                             the_cmd_arg.do_prompt = 1;
                             the_cmd_arg.style = &range_style;
                             init_arg_text (&the_cmd_arg,
                                range_name (&the_cmd_arg.val.range));
-                          }
+                        }
 			goto next_arg;
-		        }
-		      else
-		        {
+		      } else {		/* R */
 			++prompt;
 			if (get_argument (prompt, &range_style))
 			  goto new_cycle;
@@ -1767,16 +1808,18 @@ command_loop (int prefix, int iscmd)
 	    io_hide_cell_cursor ();
 	  }
 	remove_cmd_frame (frame);
+
 	/* Add frame to the list of frames to be freed on error. */
 	frame->next = running_frames;
 	running_frames = frame;
 	if (move_cursor)
 	  io_display_cell_cursor ();
+
 	if (!stub)
-	  io_error_msg ("Don't know how to invoke %s!!!",
-			frame->cmd->func_name);
+	  io_error_msg ("Don't know how to invoke %s!!!", frame->cmd->func_name);
 	else
 	  stub (frame);
+
 	running_frames = running_frames->next;
 	frame->next = 0;
 	free_cmd_frame (frame);
@@ -1843,6 +1886,11 @@ execute_command (char *str)
   int count = 1;
   ioerror = 0;
 
+#if 0
+	/* Danny */
+  fprintf(stderr, "execute_command(%s)\n", str);
+#endif
+
   /* Chop off the first word. */
   while (isspace (*str))
     ++str;
@@ -1984,6 +2032,50 @@ io_error_msg (char *str,...)
   char buf[1000];
   char buf2[1000];
 
+#if 1
+  /*
+   * Experimental : don't always crash on error.
+   */
+#ifdef	HAVE_MOTIF
+  extern int using_motif;
+
+  if (using_motif) {
+	va_start (foo, str);
+	vsprintf (buf, str, foo);
+
+	if (Global->return_from_error) {
+		Global->had_error++;		/* Indicate that we had an error */
+		MessageAppend(1, buf);
+		return;
+	}
+
+	recover_from_error ();
+	MessageAppend(1, buf);
+	longjmp (Global->error_exception, 1);
+  } else
+#endif
+  {
+  /*
+   * This is made robust against errors that occur before
+   * the io hooks have been initialized.
+   */
+  if (Global->display_opened)
+    io_bell ();
+  
+  va_start (foo, str);
+  vsprintf (buf, str, foo);
+  sprintf (buf2, "display-msg %s", buf);
+  recover_from_error ();
+
+  if (Global->display_opened)
+    execute_command (buf2);
+  else
+    fprintf (stderr, "oleo: %s\n", buf);
+
+  longjmp (Global->error_exception, 1);
+  }
+#else
+
 #ifdef	HAVE_MOTIF
   extern int using_motif;
 
@@ -1993,7 +2085,7 @@ io_error_msg (char *str,...)
 
 	recover_from_error ();
 	MessageAppend(1, buf);
-	longjmp (error_exception, 1);
+	longjmp (Global->error_exception, 1);
   } else
 #endif
   {
@@ -2001,7 +2093,7 @@ io_error_msg (char *str,...)
    * This is made robust against errors that occur before
    * the io hooks have been initialized.
    */
-  if (display_opened)
+  if (Global->display_opened)
     io_bell ();
   
   va_start (foo, str);
@@ -2009,12 +2101,14 @@ io_error_msg (char *str,...)
   sprintf (buf2, "display-msg %s", buf);
   recover_from_error ();
 
-  if (display_opened)
+  if (Global->display_opened)
     execute_command (buf2);
   else
     fprintf (stderr, "oleo: %s\n", buf);
-  longjmp (error_exception, 1);
+
+  longjmp (Global->error_exception, 1);
   }
+#endif
 }
 
 
