@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 1990, 1992, 1993, 1999 Free Software Foundation, Inc.
+ * $Id: decompile.c,v 1.15 2000/08/10 21:02:50 danny Exp $
+ *
+ * Copyright  1990, 1992, 1993, 1999 Free Software Foundation, Inc.
  * 
  * This file is part of Oleo, the GNU Spreadsheet.
  * 
@@ -22,12 +24,18 @@
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include <stdarg.h>
+#include <ctype.h>
 #include "global.h"
 #include "decompile.h"
 #include "eval.h"
 #include "cell.h"
 #include "io-utils.h"
+#include "cmd.h"
 
 struct pr_node
   {
@@ -44,12 +52,12 @@ static CELLREF decomp_col;
 /* These #defines are so that we don't have to duplicate code below */
 /* JF: now obsolete, and should be trashed. */
 
-#define F0	a0?"@%s()":"%s()"
-#define F1	a0?"@%s(%s)":"%s(%s)"
-#define F2	a0?"@%s(%s, %s)":"%s(%s, %s)"
-#define F3	a0?"@%s(%s, %s, %s)":"%s(%s, %s, %s)"
-#define F4	a0?"@%s(%s, %s, %s, %s)":"%s(%s, %s, %s, %s)"
-#define FN1	a0?"@%s(%s":"%s(%s"
+#define F0	Global->a0?"@%s()":"%s()"
+#define F1	Global->a0?"@%s(%s)":"%s(%s)"
+#define F2	Global->a0?"@%s(%s, %s)":"%s(%s, %s)"
+#define F3	Global->a0?"@%s(%s, %s, %s)":"%s(%s, %s, %s)"
+#define F4	Global->a0?"@%s(%s, %s, %s, %s)":"%s(%s, %s, %s, %s)"
+#define FN1	Global->a0?"@%s(%s":"%s(%s"
 
 /* We decompile things with these wierd node-things.  It's ugly, but it works.
  */
@@ -201,7 +209,7 @@ next_byte:
 	goto do_fn2;
 
     case C_STR:
-      tmp_str = backslash_a_string ((char *) expr + jumpto, 1);
+      tmp_str = backslash_a_string ((char *) expr + jumpto, 2);
       new = n_alloc (strlen (tmp_str) + 1,
 		     1000,
 		     "%s", tmp_str);
@@ -217,7 +225,7 @@ next_byte:
 	col = GET_COL (expr);
 	expr += EXP_ADD;
 
-	if (a0)
+	if (Global->a0)
 	  {
 	    new = n_alloc (30, 1000, f->fn_str, col_to_str (col), row);
 	  }
@@ -282,7 +290,7 @@ next_byte:
 	GET_RNG (expr, &rng);
 	expr += EXP_ADD_RNG;
 
-	if (a0)
+	if (Global->a0)
 	  new = n_alloc (40, 1000, f->fn_str, col_to_str (rng.lc), rng.lr, col_to_str (rng.hc), rng.hr);
 	else
 	  {
@@ -648,82 +656,77 @@ decomp_free (void)
 #endif
 }
 
-/* This takes a string and returns a backslashed form suitable for printing.
-   Iff add_quote is true, it'll add "s at the beginning and end.
-   Note that this returns a pointer to a static area that is overwritten with
-   each call. . .
+/*
+ * This takes a string and returns a backslashed form suitable for printing.
+ * Iff add_quote is true, it'll add "s at the beginning and end.
+ * Note that this returns a pointer to a static area that is overwritten with
+ * each call. . .
+ *
+ * If add_quote is 1, escape normal quotes, if it is 2, also escape %.
  */
 char *
 backslash_a_string (char *string, int add_quote)
 {
-  char *pf;
-  char *pt;
-  int ch;
-  int size;
-  int len;
-
-  static char *cbuf;
-  static int s_cbuf;
+	char		*pf, *pt;
+	unsigned char	ch;
+	int		size, len;
+	static char	*cbuf;
+	static int	s_cbuf;
 
 #define ALLOC_PT()				\
 	len=strlen(pf);				\
 	size=pf-string;				\
 	if(s_cbuf<3+size+4*len) {		\
-		s_cbuf=3+size+4*len;		\
-		cbuf= (cbuf) ? ck_realloc(cbuf,s_cbuf) : ck_malloc(s_cbuf); \
+	s_cbuf=3+size+4*len;		\
+	cbuf= (cbuf) ? ck_realloc(cbuf,s_cbuf) : ck_malloc(s_cbuf); \
 	}					\
 	if(size)				\
-		bcopy(string,cbuf,size);	\
+	bcopy(string,cbuf,size);	\
 	pt=cbuf+size;				\
 
 
-  pt = 0;
-  pf = string;
-  if (add_quote)
-    {
-      ALLOC_PT ()
-	* pt++ = '"';
-    }
-  for (; *pf; pf++)
-    {
-      ch = *pf;
-      if (ch >= ' ' && ch <= '~' && ch != '\\' && (ch != '"' || !add_quote))
-	{
-	  if (pt)
-	    *pt++ = ch;
-	  continue;
+	pt = 0;
+	pf = string;
+	if (add_quote) {
+		ALLOC_PT ()
+		*pt++ = '"';
 	}
+	for (; *pf; pf++) {
+		ch = *pf;
+		if (isprint(ch) && ch != '\\'
+				&& (ch != '%' || !(add_quote == 2))
+				&& (ch != '"' || !add_quote)) {
+			if (pt)
+				*pt++ = ch;
+			continue;
+		}
 
-      if (!pt)
-	{
-	  ALLOC_PT ()
+		if (!pt) {
+			ALLOC_PT ()
+		}
+		if (ch == '\\') {
+			*pt++ = '\\';
+			*pt++ = '\\';
+		} else if (ch == '"') {
+			*pt++ = '\\';
+			*pt++ = ch;
+		} else if (ch == '%') {
+			*pt++ = '%';
+			*pt++ = '%';
+		} else {
+			*pt++ = '\\';
+			*pt++ = ((ch >> 6) & 0x3) + '0';
+			*pt++ = ((ch >> 3) & 0x7) + '0';
+			*pt++ = (ch & 0x7) + '0';
+		}
 	}
-      if (ch == '\\')
-	{
-	  *pt++ = '\\';
-	  *pt++ = '\\';
+	if (add_quote)
+		*pt++ = '"';
+	if (pt) {
+		*pt++ = '\0';
+		return cbuf;
 	}
-      else if (ch == '"')
-	{
-	  *pt++ = '\\';
-	  *pt++ = ch;
-	}
-      else
-	{
-	  *pt++ = '\\';
-	  *pt++ = ((ch >> 6) & 0x3) + '0';
-	  *pt++ = ((ch >> 3) & 0x7) + '0';
-	  *pt++ = (ch & 0x7) + '0';
-	}
-    }
-  if (add_quote)
-    *pt++ = '"';
-  if (pt)
-    {
-      *pt++ = '\0';
-      return cbuf;
-    }
-  return string;
+	return string;
 }
 
 void
