@@ -1,3 +1,30 @@
+/*
+ * $Id: defun.h,v 1.7 2000/08/10 21:02:50 danny Exp $
+ *
+ * Copyright  1993, 2000 Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this software; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
+
+/*
+ * This file is include multiple times in defuns.h to do all kinds of
+ * crazy things.
+ *
+ * Lousy design :-(
+ */
+
 /* Commands to Stop and to Suspend the Beast */
 
 DEFUN ("kill-oleo", FDkill_oleo, FAkill_oleo, kill_oleo)
@@ -1467,223 +1494,6 @@ DEFUN ("write-region-to-file",
 "inside of RANGE.",
 #endif
 
-/* Graph Commands */
-
-#ifndef X_DISPLAY_MISSING
-DEFUN_3 ("graph-x11-mono", FDgraph_x11_mono, graph_x11_mono)
-#ifdef DOC_STRINGS
-"Specify that graphing should be done to an X11 window in black and white.",
-#endif
-#endif
-
-#ifndef X_DISPLAY_MISSING
-DEFUN_3 ("graph-x11-color", FDgraph_x11_color, graph_x11_color)
-#ifdef DOC_STRINGS
-"Specify that graphing should be done to an X11 window in color.",
-#endif
-#endif
-
-DEFUN ("graph-toggle-axis-logness",
-       FDgraph_toggle_x_logness, FAgraph_toggle_x_logness, graph_set_logness)
-#ifdef FUNC_ARG_STRINGS
-"[xXyY]Which axis? (`x' or `y') ",
-"p?",
-"p",
-#endif
-#ifdef DOC_STRINGS
-"Toggle whether AXIS (`x' or `y') is drawn to a logarithmic scale.",
-"With a 0 or negative prefix, make the axis not be logarithmic.",
-"With a positive prefix, make the axis logarithmic.",
-#endif
-
-DEFUN ("graph-postscript",
-       FDgraph_postscript, FAgraph_postscript, graph_postscript)  
-#ifdef FUNC_ARG_STRINGS
-"fnWrite postscript graph to? ",
-"[ledLED][L]andscape  [E]mbedded postscript  or [D]efault",
-"[cCmM][C]olor  or [M]onochrome",
-"sPostscript font? ", 
-"nPoint size? ",
-#endif
-#ifdef DOC_STRINGS
-"Specify that graphing should be done in postscript.",
-"Arguments are:",
-"  FILE - the output file for the postscript (may be a pipe, e.g.: `| lpr').",
-"  KIND - [l]andscape, [e]mbedded postscript or [d]efault",
-"  COLOR - [b]lack and white,  or [c]olor",
-"  FONT - a font name (e.g.: Times-Roman)",
-"  PTSIZ - point size for labels.",
-#endif
-
-DEFUN ("graph-set-axis-low",
-       FDgraph_set_axis_low, FAgraph_set_axis_low, graph_set_axis_lo)
-#ifdef FUNC_ARG_STRINGS
-"[xXyY]Set range start for which axis? (`x' or `y') ",
-"sBeginning of the %0 axis range [a number or `def']? ",
-#endif
-#ifdef DOC_STRINGS
-"Set the beginning of the range graph on AXIS to LOW.",
-"LOW may be either a real number, or `def' to invoke auto-scaling.",
-#endif
-
-DEFUN ("graph-set-axis-high",
-       FDgraph_set_axis_high, FAgraph_set_axis_high, graph_set_axis_hi)
-#ifdef FUNC_ARG_STRINGS
-"[xXyY]Set range start for which axis? (`x' or `y') ",
-"sEnd of the %0 axis range [a number or `def']? ",
-#endif
-#ifdef DOC_STRINGS
-"Set the beginning of the range graph on AXIS to HIGH.",
-"HIGH may be either a real number, or `def' to invoke auto-scaling.",
-#endif
-
-DEFUN ("graph-set-x-axis-symbolic", FDgraph_set_axis_symbolic,
-       FAgraph_set_axis_symbolic, graph_set_axis_symbolic)
-#ifdef FUNC_ARG_STRINGS
-"c#x",
-"rSymbols for the %0-axis in range? ",
-"cRead rows or cols in %1? (`r' or `c') ",
-#endif
-#ifdef DOC_STRINGS
-"Declare that the x-axis should range over the names in RANGE read in ORDER.",
-"RANGE should be a set of cells containing the symbols for the x-axis.",
-"The values in those cells correspond to positions in the range being",
-"graphed (e.g.: the cells might contain the names of the months).",
-"ORDER (either `r' or `c') says whether cells should be read row-wise",
-"or column-wise.",
-"",
-"After this command, two kinds of data-sets can be plotted.",
-"Datasets which are explicitly made up of pairs of cells should contain",
-"X values that can be found in RANGE.  Datasets with an implicit X coordinate",
-"will be provided with X coordinates from RANGE in the specified order.",
-"",
-"For non-symbolic ranges, use [graph-set-axis-low]",
-"",
-"For symbolic labels on a numeric range, use [graph-set-axis-labels]",
-#endif
-
-DEFUN ("graph-set-axis-labels", FDgraph_set_axis_labels,
-       FAgraph_set_axis_labels, graph_set_axis_labels)
-#ifdef FUNC_ARG_STRINGS
-"[xXyY]Labels for which axis? (`x' or `y') ",
-"rRead labels for the %0-axis from range? ",
-"[hHvViI]Are position/label pairs [H]oriz, [V]ert, or [I]mplicit-position? ",
-"[rRcC]Read labels from %0 row-wise or column-wise? (`r',or `c') ",
-#endif
-#ifdef DOC_STRINGS
-"Read labels for AXIS from RNG as ORIENTATION pos/label pairs read in ORDER.",
-"",
-"AXIS may be either `x' or `y'.",
-"",
-"ORIENTATION may be:",
-"	`h' - horizontal	if rc contains the position, then rc[1]",
-"				contains the label.",
-"	`v' - vertical		if rc contains the position, then r[1]c",
-"				contains the label.",
-"	`i' - implicit		every cell is a label; positions are assigned",
-"				non-negative integer positions (0, 1, 2...)",
-"",
-"ORDER may be:",
-"	`r' - rows		pairs are read row-wise",
-"	`c' - columns		pairs are read column-wise",
-"",
-"This command does not effect what range will be graphed.  If labels are",
-"assigned positions outside the visible range, they will not be drawn.",
-"See also graph-set-x-symbolic.",
-"This is undone by [graph-default-axis-labels]",
-#endif
-
-DEFUN ("graph-default-axis-labels", FAgraph_default_axis_labels,
-       FDgraph_default_axis_labels, graph_default_axis_labels)
-#ifdef FUNC_ARG_STRINGS
-"[xXyY]Use default labels for which axis? (`x' or `y') ",
-#endif
-#ifdef DOC_STRINGS
-"Reset the labels for AXIS to their defaults.",
-"This undoes [graph-set-axis-labels]",
-#endif
-
-DEFUN ("graph-set-style",
-       FDgraph_set_style, FAgraph_set_style, graph_set_style)
-#ifdef FUNC_ARG_STRINGS
-"N[0,9]Set style for which dataset? (0..9) ",
-"sStyle for dataset %0? ",
-#endif
-#ifdef DOC_STRINGS
-"Specify that dataset N should be graphed in STYLE.",
-"STYLE may be any of the gnuplot graph types: ",
-"	lines, points, linespoints, impulses, dots",
-#endif
-
-DEFUN ("graph-set-axis-title",
-       FDgraph_set_axis_title, FAgraph_set_axis_title, graph_set_axis_title)
-#ifdef FUNC_ARG_STRINGS
-"[xyXY]Set title of which axis? (x or y) ",
-"sTitle for the %0-axis? ",
-#endif
-#ifdef DOC_STRINGS
-"When drawing graphs, label AXIS (`x' or `y') with TITLE.",
-#endif
-
-DEFUN ("graph-set-data", FDgraph_set_data, FAgraph_set_data, graph_set_data)
-#ifdef FUNC_ARG_STRINGS
-"N[0,9]Set which dataset? (0..9) ",
-"rRead dataset %0 from what range? ",
-"[hHvViI]Are X,Y pairs [H]oriz, [V]ert, or [I]mplicit-X? ",
-"[rRcC]Read labels from %0 row-wise or column-wise? (`r',or `c') ",
-#endif
-#ifdef DOC_STRINGS
-"Specify that dataset N is read from RNG as ORNT x,y pairs read in ORDER",
-"",
-"Datasets are numbered 0..9.",
-"",
-"ORIENTATION may be:",
-"	`h' - horizontal	if rc contains x, then rc[1] contains y",
-"	`v' - vertical		if rc    \"     x,   \"  r[1]c    \"     y",
-"	`i' - implicit		every cell is a y value; x coordinates are",
-"				non-negative integer positions (0, 1, 2...)",
-"",
-"ORDER may be:",
-"	`r' - rows		pairs are read row-wise",
-"	`c' - columns		pairs are read column-wise",
-"",
-#endif
-
-DEFUN ("graph-set-data-title",
-       FDgraph_set_data_title, FAgraph_set_data_title, graph_set_data_title)
-#ifdef FUNC_ARG_STRINGS
-"N[0,9]Set the title for which dataset? (0..9) ",
-"sTitle for dataset %0? ",
-#endif
-#ifdef DOC_STRINGS
-"Specify that dataset N should be called TITLE. ",
-"The title will be printed in the key for the graph.",
-#endif
-
-DEFUN_3 ("graph-presets", FDgraph_presets, graph_presets)
-#ifdef DOC_STRINGS
-"Return graphing parameters to their default values.",
-"This does not effect the datasets selected for graphing.",
-#endif
-
-DEFUN_3 ("graph-clear-datasets", FDgraph_clear_datasets, graph_clear_datasets)
-#ifdef DOC_STRINGS
-"Erase the settings of which datasets will be graphed.",
-"This undoes the effects of all previous invocations of [graph-set-data].",
-#endif
-
-DEFUN_3 ("graph-make-info", FDgraph_make_info, graph_make_info)  
-#ifdef DOC_STRINGS
-"Update the info-buffer called `graphing-parameters'.",
-"The new contents of the buffer will describe the current settings",
-"of all graphing parameters",
-#endif
-
-DEFUN_3 ("graph-plot", FDgraph_plot, graph_plot)
-#ifdef DOC_STRINGS
-"Execute a graph.  The parameter of the graph are set using [graph-setup]",
-#endif
-
 /* Setting Font Names for PostScript Printing and X11 */
 
 DEFUN ("define-font", FDdefine_font, FAdefine_font, define_font)
@@ -2392,3 +2202,220 @@ DEFUN ("page-info-backwards",
 "Move backward a page in an info text.  With a prefix, move that many pages.",
 #endif
 
+#ifdef	HAVE_LIBPLOT
+
+/*
+ * Plotutils commands
+ */
+
+DEFUN_3 ("plotutils-metaplot", FDplotutils_metaplot, plotutils_metaplot)
+#ifdef DOC_STRINGS
+"Specify that graphing output should be in GNU MetaPlot format.",
+#endif
+
+DEFUN_3 ("plotutils-illustrator", FDplotutils_illustrator, plotutils_illustrator)
+#ifdef DOC_STRINGS
+"Specify that graphing output should be in Adobe Illustrator format.",
+#endif
+
+DEFUN_3 ("plotutils-fig", FDplotutils_fig, plotutils_fig)
+#ifdef DOC_STRINGS
+"Specify that graphing output should be in FIG format.",
+#endif
+
+DEFUN_3 ("plotutils-x-color", FDplotutils_x_color, plotutils_x_color)
+#ifdef DOC_STRINGS
+"Specify that graphing output should be in X COLOR format.",
+#endif
+
+DEFUN_3 ("plotutils-x-mono", FDplotutils_x_mono, plotutils_x_mono)
+#ifdef DOC_STRINGS
+"Specify that graphing output should be in X MONO format.",
+#endif
+
+DEFUN_3 ("plotutils-gif", FDplotutils_gif, plotutils_gif)
+#ifdef DOC_STRINGS
+"Specify that graphing output should be in GIF format.",
+#endif
+
+DEFUN_3 ("plotutils-png", FDplotutils_png, plotutils_png)
+#ifdef DOC_STRINGS
+"Specify that graphing output should be in PNG format.",
+#endif
+
+DEFUN_3 ("plotutils-pcl", FDplotutils_pcl, plotutils_pcl)
+#ifdef DOC_STRINGS
+"Specify that graphing should be done to a PCL printer.",
+#endif
+
+DEFUN_3 ("plotutils-hpgl", FDplotutils_hpgl, plotutils_hpgl)
+#ifdef DOC_STRINGS
+"Specify that graphing should be done to a HP-GL printer or plotter.",
+#endif
+
+DEFUN_3 ("plotutils-tek", FDplotutils_tek, plotutils_tek)
+#ifdef DOC_STRINGS
+"Specify that graphing should be done to a Tektronix terminal (or xterm emulator).",
+#endif
+
+DEFUN_3 ("plotutils-regis", FDplotutils_regis, plotutils_regis)
+#ifdef DOC_STRINGS
+"Specify that graphing should be done to a ReGIS terminal.",
+#endif
+
+DEFUN_3 ("plotutils-bar", FDplotutils_bar, plotutils_bar)
+#ifdef DOC_STRINGS
+"Specify that the graph is a bar chart.",
+#endif
+
+DEFUN_3 ("plotutils-pie", FDplotutils_pie, plotutils_pie)
+#ifdef DOC_STRINGS
+"Specify that the graph is a pie chart.",
+#endif
+
+DEFUN_3 ("plotutils-xy", FDplotutils_xy, plotutils_xy)
+#ifdef DOC_STRINGS
+"Specify that the graph is an xy chart.",
+#endif
+
+DEFUN ("plotutils-postscript",
+       FDplotutils_postscript, FAplotutils_postscript, graph_postscript)  
+#ifdef FUNC_ARG_STRINGS
+"fnWrite postscript graph to? ",
+"[ledLED][L]andscape  [E]mbedded postscript  or [D]efault",
+"[cCmM][C]olor  or [M]onochrome",
+"sPostscript font? ", 
+"nPoint size? ",
+#endif
+#ifdef DOC_STRINGS
+"Specify that graphing should be done in postscript.",
+"Arguments are:",
+"  FILE - the output file for the postscript (may be a pipe, e.g.: `| lpr').",
+"  KIND - [l]andscape, [e]mbedded postscript or [d]efault",
+"  COLOR - [b]lack and white,  or [c]olor",
+"  FONT - a font name (e.g.: Times-Roman)",
+"  PTSIZ - point size for labels.",
+#endif
+
+DEFUN("plotutils-set-filename",
+	FDplotutils_set_filename, FAplotutils_set_filename, plotutils_set_filename)
+#ifdef FUNC_ARG_STRINGS
+"fnWrite graph to? ",
+#endif
+#ifdef DOC_STRINGS
+"Specify to which file graphs should be written.",
+"  FILE - the output file for the postscript (may be a pipe, e.g.: `| lpr').",
+"         (may be a dash '-' for indicating standard output)",
+#endif
+
+DEFUN ("plotutils-set-style",
+       FDplotutils_set_style, FAplotutils_set_style, graph_set_style)
+#ifdef FUNC_ARG_STRINGS
+"N[0,9]Set style for which dataset? (0..9) ",
+"sStyle for dataset %0? ",
+#endif
+#ifdef DOC_STRINGS
+"Specify that dataset N should be graphed in STYLE.",
+"STYLE may be any of the gnuplot graph types: ",
+"	lines, points, linespoints, impulses, dots",
+#endif
+
+DEFUN ("plotutils-set-data", FDplotutils_set_data, FAplotutils_set_data, plotutils_set_data)
+#ifdef FUNC_ARG_STRINGS
+"N[0,9]Set which dataset? (0..9) ",
+"rRead dataset %0 from what range? ",
+/* "[hHvViI]Are X,Y pairs [H]oriz, [V]ert, or [I]mplicit-X? ", */
+/* "[rRcC]Read labels from %0 row-wise or column-wise? (`r',or `c') ", */
+#endif
+#ifdef DOC_STRINGS
+"Specify that dataset N is read from RNG",
+"",
+"Datasets are numbered 0..9.",
+"",
+#endif
+
+DEFUN ("plotutils-set-data-title",
+       FDplotutils_set_data_title, FAplotutils_set_data_title, graph_set_data_title)
+#ifdef FUNC_ARG_STRINGS
+"N[0,9]Set the title for which dataset? (0..9) ",
+"sTitle for dataset %0? ",
+#endif
+#ifdef DOC_STRINGS
+"Specify that dataset N should be called TITLE. ",
+"The title will be printed in the key for the graph.",
+#endif
+
+DEFUN_3 ("plotutils-plot", FDplotutils_plot, graph_plot)
+#ifdef DOC_STRINGS
+"Execute a graph.  The parameter of the graph are set using [graph-setup]",
+#endif
+
+DEFUN_3 ("plotutils-make-info", FDplotutils_make_info, plotutils_make_info)  
+#ifdef DOC_STRINGS
+"Update the info-buffer called `graphing-parameters'.",
+"The new contents of the buffer will describe the current settings",
+"of all graphing parameters",
+#endif
+
+DEFUN ("plotutils-set-axis-low",
+       FDplotutils_set_axis_low, FAplotutils_set_axis_low, graph_set_axis_lo)
+#ifdef FUNC_ARG_STRINGS
+"[xXyY]Set range start for which axis? (`x' or `y') ",
+"sBeginning of the %0 axis range [a number or `def']? ",
+#endif
+#ifdef DOC_STRINGS
+"Set the beginning of the range graph on AXIS to LOW.",
+"LOW may be either a real number, or `def' to invoke auto-scaling.",
+#endif
+
+DEFUN ("plotutils-set-axis-high",
+       FDplotutils_set_axis_high, FAplotutils_set_axis_high, graph_set_axis_hi)
+#ifdef FUNC_ARG_STRINGS
+"[xXyY]Set range start for which axis? (`x' or `y') ",
+"sEnd of the %0 axis range [a number or `def']? ",
+#endif
+#ifdef DOC_STRINGS
+"Set the beginning of the range graph on AXIS to HIGH.",
+"HIGH may be either a real number, or `def' to invoke auto-scaling.",
+#endif
+
+DEFUN ("plotutils-default-axis-labels", FAplotutils_default_axis_labels,
+       FDplotutils_default_axis_labels, graph_default_axis_labels)
+#ifdef FUNC_ARG_STRINGS
+"[xXyY]Use default labels for which axis? (`x' or `y') ",
+#endif
+#ifdef DOC_STRINGS
+"Reset the labels for AXIS to their defaults.",
+"This undoes [graph-set-axis-labels]",
+#endif
+
+DEFUN ("plotutils-set-axis-labels", FDplotutils_set_axis_labels,
+       FAplotutils_set_axis_labels, plotutils_set_axis_labels)
+#ifdef FUNC_ARG_STRINGS
+"[xXyY]Labels for which axis? (`x' or `y') ",
+"rRead labels for the %0-axis from range? ",
+#endif
+#ifdef DOC_STRINGS
+"Read labels for AXIS from RNG.",
+"",
+"AXIS may be either `x' or `y'.",
+"",
+"This command does not effect what range will be graphed.  If labels are",
+"assigned positions outside the visible range, they will not be drawn.",
+"See also graph-set-x-symbolic.",
+"This is undone by [graph-default-axis-labels]",
+#endif
+
+DEFUN_3 ("plotutils-presets", FDplotutils_presets, graph_presets)
+#ifdef DOC_STRINGS
+"Return graphing parameters to their default values.",
+"This does not effect the datasets selected for graphing.",
+#endif
+
+DEFUN_3 ("plotutils-clear-datasets", FDplotutils_clear_datasets, graph_clear_datasets)
+#ifdef DOC_STRINGS
+"Erase the settings of which datasets will be graphed.",
+"This undoes the effects of all previous invocations of [graph-set-data].",
+#endif
+
+#endif	/* ~ HAVE_LIBPLOT */
