@@ -1,5 +1,7 @@
 /*
- * hash.c - hash table lookup strings - Copyright (C) 1987, 1992, 1993 Free Software
+ * $Id: hash.c,v 1.5 2000/08/10 21:02:50 danny Exp $
+ *
+ * hash.c - hash table lookup strings - Copyright  1987, 1992, 1993 Free Software
  * Foundation, Inc.
  *
  * This file is part of GAS, the GNU Assembler.
@@ -118,12 +120,14 @@ of char[size].  Free(address) frees same.  */
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include <stdio.h>
 #ifndef VOIDSTAR
 #define VOIDSTAR void *
 #endif
-VOIDSTAR malloc ();
-void free ();
 
 #undef TRUE
 #define TRUE           (1)
