@@ -1,23 +1,31 @@
-/*	Copyright (C) 1993 Free Software Foundation, Inc.
+/*
+ * $Id: init.c,v 1.7 2000/08/10 21:02:50 danny Exp $
+ *
+ * Copyright  1993, 2000 Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this software; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this software; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "proto.h"
 #include "init.h"
 #include "global.h"
@@ -464,6 +472,96 @@ char *init_cmds[] =
   "bind-key generic-menu exit-minibuffer ^m",
   "bind-key generic-menu exit-minibuffer ^j",
   "",
+#ifdef	HAVE_LIBPLOT
+  "# Graphing with GNU Plotutils",
+  "",
+  "create-keymap plotutils-commands generic-menu",
+  "name-macro-string plotutils-setup \\",
+  " {with-keymap plotutils-commands}{builtin-help Plotutils-graphs}",
+  "bind-key generic-meta plotutils-setup g",
+  "set-map-prompt plotutils-commands \\",
+  "(R C o d v, p x y t s f, X P B Esc)? ",
+  "",
+  "bind-key plotutils-commands plotutils-plot p",
+  "bind-key plotutils-commands plotutils-set-data-title t",
+  "bind-key plotutils-commands plotutils-set-data d",
+  "bind-key plotutils-commands plotutils-set-style s",
+  "bind-key plotutils-commands plotutils-set-filename f",
+  "",
+  "bind-key plotutils-commands plotutils-xy X",
+  "bind-key plotutils-commands plotutils-pie P",
+  "bind-key plotutils-commands plotutils-bar B",
+  "",
+  "name-macro-string plotutils-preset-msg {display-msg Graph reset}",
+  "name-macro-string plotutils-presets-verbosely \\",
+  "{plotutils-presets}{plotutils-preset-msg}{set-info _expanded__info_graphing}",
+  "bind-key plotutils-commands plotutils-presets-verbosely R",
+  "",
+  "name-macro-string plotutils-clear-msg {display-msg Graph data reset}",
+  "name-macro-string plotutils-clear-verbosely \\",
+  "{plotutils-clear-datasets}{plotutils-clear-msg}{set-info _expanded__info_graphing}",
+  "bind-key plotutils-commands plotutils-clear-verbosely C",
+  "",
+  "create-keymap plotutils-terminal-types universal",
+  "name-macro-string plotutils-select-output \\",
+  "{one-command-with-keymap plotutils-terminal-types}{builtin-help plotutils-output-types}",
+  "bind-key plotutils-commands plotutils-select-output o",
+  "set-map-prompt plotutils-terminal-types \\",
+  "Graph output type (p t r m i f P h g n x X): ",
+  "bind-key plotutils-terminal-types plotutils-postscript p",
+  "bind-key plotutils-terminal-types plotutils-tek t",
+  "bind-key plotutils-terminal-types plotutils-regis r",
+  "bind-key plotutils-terminal-types plotutils-metaplot m",
+  "bind-key plotutils-terminal-types plotutils-illustrator i",
+  "bind-key plotutils-terminal-types plotutils-fig f",
+  "bind-key plotutils-terminal-types plotutils-pcl P",
+  "bind-key plotutils-terminal-types plotutils-hpgl h",
+  "bind-key plotutils-terminal-types plotutils-gif g",
+  "bind-key plotutils-terminal-types plotutils-png n",
+  "bind-key plotutils-terminal-types plotutils-x-color X",
+  "bind-key plotutils-terminal-types plotutils-x-mono x",
+  "",
+  "create-keymap view-plotutils-parameters view-info",
+  "name-macro-string plotutils-verify \\",
+  "{plotutils-make-info}{with-keymap view-plotutils-parameters}{set-info graphing-parameters}",
+  "bind-key plotutils-commands plotutils-verify v",
+  "set-map-prompt view-plotutils-parameters \\",
+  "+ (forward page)   - (backward page)   p (plot)  or  ESC",
+  "bind-key view-plotutils-parameters plotutils-plot p",
+  "",
+  "create-keymap plotutils-x-axis-commands generic-menu",
+  "name-macro-string plotutils-x-axis \\",
+  "{with-keymap plotutils-x-axis-commands}{builtin-help plotutils-x-axis-help}",
+  "bind-key plotutils-commands plotutils-x-axis x",
+  "set-map-prompt plotutils-x-axis-commands X-axis (s [ ] l L Esc)",
+  "",
+  "name-macro-string plotutils-set-x-range-low {plotutils-set-axis-low x}",
+  "name-macro-string plotutils-set-x-range-high {plotutils-set-axis-high x}",
+  "bind-key plotutils-x-axis-commands plotutils-set-x-range-low [",
+  "bind-key plotutils-x-axis-commands plotutils-set-x-range-high ]",
+  "name-macro-string plotutils-set-x-labels {plotutils-set-axis-labels x}",
+  "bind-key plotutils-x-axis-commands plotutils-set-x-labels l",
+  "name-macro-string plotutils-default-x-labels {plotutils-default-axis-labels x}",
+  "bind-key plotutils-x-axis-commands plotutils-default-x-labels L",
+  "",
+  "create-keymap plotutils-y-axis-commands generic-menu",
+  "name-macro-string plotutils-y-axis \\",
+  "{with-keymap plotutils-y-axis-commands}{builtin-help plotutils-y-axis-help}",
+  "bind-key plotutils-commands plotutils-y-axis y",
+  "set-map-prompt plotutils-y-axis-commands Y-axis (s [ ] l L Esc)",
+  "",
+  "name-macro-string plotutils-set-y-range-low {plotutils-set-axis-low y}",
+  "name-macro-string plotutils-set-y-range-high {plotutils-set-axis-high y}",
+  "bind-key plotutils-y-axis-commands plotutils-set-y-range-low [",
+  "bind-key plotutils-y-axis-commands plotutils-set-y-range-high ]",
+  "name-macro-string plotutils-set-y-labels {plotutils-set-axis-labels y}",
+  "bind-key plotutils-y-axis-commands plotutils-set-y-labels l",
+  "name-macro-string plotutils-default-y-labels {plotutils-default-axis-labels y}",
+  "bind-key plotutils-y-axis-commands plotutils-default-y-labels L",
+  "",
+#else
+/* Don't use gnuplot if you also have plotutils */
+# ifdef	WITH_GNUPLOT
   "# Graphing with gnuplot",
   "",
   "create-keymap graph-commands generic-menu",
@@ -478,7 +576,7 @@ char *init_cmds[] =
   "{graph-presets}{graph-preset-msg}{set-info _expanded__info_graphing}",
   "bind-key graph-commands graph-presets-verbosely R",
   "",
-  "name-macro-string graph-clear-msg {display-error-msg Graph data reset}",
+  "name-macro-string graph-clear-msg {display-msg Graph data reset}",
   "name-macro-string graph-clear-verbosely \\",
   "{graph-clear-datasets}{graph-clear-msg}{set-info _expanded__info_graphing}",
   "bind-key graph-commands graph-clear-verbosely C",
@@ -502,10 +600,6 @@ char *init_cmds[] =
   "bind-key graph-commands graph-select-output o",
   "set-map-prompt graph-terminal-types \\",
   "Graph output type (x X P): ",
-#ifndef X_DISPLAY_MISSING
-  "bind-key graph-terminal-types graph-x11-mono x",
-  "bind-key graph-terminal-types graph-x11-color X",
-#endif
   "bind-key graph-terminal-types graph-postscript p",
   "",
   "create-keymap graph-x-axis-commands generic-menu",
@@ -539,6 +633,8 @@ char *init_cmds[] =
   "name-macro-string graph-default-y-labels {graph-default-axis-labels y}",
   "bind-key graph-y-axis-commands graph-default-y-labels L",
   "",
+# endif	/* WITH_GNUPLOT */
+#endif	/* ~ HAVE_LIBPLOT */
   "# printing",
   "",
   "create-keymap print-commands universal",
