@@ -1,24 +1,32 @@
-/*	Copyright (C) 1992, 1993 Free Software Foundation, Inc.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this software; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+/*
+ * $Id: io-curses.c,v 1.17 2000/08/10 21:02:50 danny Exp $
+ *
+ * Copyright  1992, 1993, 1999 Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this software; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "proto.h"
 #include "funcdef.h"
 #include <stdio.h>
@@ -60,7 +68,6 @@ static int textout = 0;
 static int term_cursor_claimed = 0;
 
 static void move_cursor_to (struct window *, CELLREF, CELLREF, int);
-
 
 static int
 curses_metric (char * str, int len)
@@ -75,7 +82,7 @@ static void
 _io_redraw_input (void)
 {
   int pos;
-  int row = (input_view.current_info ? 0 : input);
+  int row = (input_view.current_info ? 0 : Global->input);
 
   if (input_view.info_redraw_needed)
     {
@@ -175,10 +182,6 @@ _io_over (char * str, int len)
   iv_over (&input_view, len);
 }
 
-
-
-
-
 static void 
 _io_display_cell_cursor (void)
 {
@@ -258,8 +261,6 @@ _io_hide_cell_cursor (void)
 #endif
   move (y, x);
 }
-
-
 
 /* Functions, etc for dealing with cell contents being displayed
 	on top of other cells. */
@@ -368,7 +369,6 @@ change_slop (VOIDSTAR where,
 	}
     }
 }
-
 
 static void 
 _io_open_display (void)
@@ -437,7 +437,7 @@ _io_redisp (void)
 	    input_view.expanded_keymap_prompt))
 	move_cursor_to (cwin, curow, cucol, 0);
       else
-	move ((input_view.current_info ? 0 : input), 
+	move ((input_view.current_info ? 0 : Global->input), 
 	      input_view.prompt_wid + input_view.input_cursor -
 	      input_view.visibility_begin); 
     }
@@ -496,7 +496,7 @@ _io_repaint (void)
 		  char *ptr;
 		  char buf[30];
 
-		  if (a0)
+		  if (Global->a0)
 		    ptr = col_to_str (cc);
 		  else
 		    {
@@ -526,7 +526,7 @@ _io_repaint (void)
 	      if (n1)
 		{
 		  move (n, win->win_over - win->lh_wid);
-		  if (a0)
+		  if (Global->a0)
 		    printw ("%-*d ", win->lh_wid - 1, rr);
 		  else
 		    printw ("R%-*d", win->lh_wid - 1, rr);
@@ -552,13 +552,16 @@ _io_repaint (void)
 }
 
 static void 
-_io_close_display (void)
+_io_close_display (int e)
 {
-  clear ();
-  refresh ();
-  (void) endwin ();
+	if (e == 0) {
+		clear ();
+		refresh ();
+	}
+
+	(void) endwin ();
 }
-
+
 static int 
 _io_input_avail (void)
 {
@@ -588,7 +591,7 @@ _io_read_kbd (char *buf, int size)
   FD_CLR (0, &exception_pending_fd_set);
   return r;
 }
-
+
 
 #if defined(SIGIO)
 
@@ -617,15 +620,14 @@ _io_getch (void)
 	  ? EOF
 	  : ch);
 }
-
 
 static int 
 _io_get_chr (char *prompt)
 {
   int x;
-  mvaddstr (input, 0, prompt);
+  mvaddstr (Global->input, 0, prompt);
   clrtoeol ();
-  topclear = 2;
+  Global->topclear = 2;
   refresh ();
   ++term_cursor_claimed;
   x = get_chr ();
@@ -635,19 +637,6 @@ _io_get_chr (char *prompt)
 
 #define BUFFER 10
 
-static void 
-local_putchar (int ch)
-{
-  (void) putc (ch, stdout);
-  /* (void)putchar(ch); */
-}
-
-static void
-local_puts (char *s)
-{
-   (void) tputs (s, 1, (int (*)()) local_putchar);
-}
-
 static void 
 _io_bell (void)
 {
@@ -673,7 +662,6 @@ _io_bell (void)
 #endif
 }
 
-
 static void
 move_cursor_to (struct window *win, CELLREF r, CELLREF c, int dn)
 {
@@ -706,7 +694,7 @@ _io_update_status (void)
   if (!user_status || input_view.current_info)
     return;
   getyx (stdscr, yy, xx);
-  move (status, 0);
+  move (Global->status, 0);
   wid = COLS - 2;
   
   if (mkrow != NON_ROW)
@@ -778,11 +766,11 @@ static void
 _io_clear_input_before (void)
 {
   textout = 0;
-  if (topclear == 2)
+  if (Global->topclear == 2)
     {
-      move (input, 0);
+      move (Global->input, 0);
       clrtoeol ();
-      topclear = 0;
+      Global->topclear = 0;
     }
   move (0, 0);
 }
@@ -790,11 +778,11 @@ _io_clear_input_before (void)
 static void
 _io_clear_input_after (void)
 {
-  if (topclear)
+  if (Global->topclear)
     {
-      move (input, 0);
+      move (Global->input, 0);
       clrtoeol ();
-      topclear = 0;
+      Global->topclear = 0;
     }
 }
 
@@ -980,7 +968,7 @@ _io_pr_cell_win (struct window *win, CELLREF r, CELLREF c, CELL *cp)
       else
 	set_slop ((VOIDSTAR *) (&(win->win_slops)), r, c, cc);
     }
-  if ((hgt > 1) && display_formula_mode)
+  if ((hgt > 1) && Global->display_formula_mode)
     {
       move_cursor_to (win, r, c, 1);
       ptr = decomp (r, c, cp);
@@ -991,7 +979,6 @@ _io_pr_cell_win (struct window *win, CELLREF r, CELLREF c, CELL *cp)
     io_update_status ();
   move (yy, xx);
 }
-
 
 
 static void
@@ -1000,8 +987,6 @@ _io_flush (void)
   refresh ();
 }
 
-
-
 void
 _io_command_loop (int a)
 {
