@@ -1,30 +1,39 @@
-/*	Copyright (C) 1990, 1992, 1993 Free Software Foundation, Inc.
+/*
+ * $Id: io-term.c,v 1.45 2000/11/22 19:33:01 danny Exp $
+ *
+ * Copyright  1990, 1992, 1993, 1999, 2000 Free Software Foundation, Inc.
+ * 
+ * This file is part of Oleo, the GNU Spreadsheet.
+ * 
+ * Oleo is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ * 
+ * Oleo is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Oleo; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
-This file is part of Oleo, the GNU Spreadsheet.
-
-Oleo is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-Oleo is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Oleo; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+static char *rcsid = "$Id: io-term.c,v 1.45 2000/11/22 19:33:01 danny Exp $";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <libintl.h>
-#include <locale.h>
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
 
 #include <ctype.h>
 #include <stdio.h>
+#include <locale.h>
+#include <stdlib.h>
 
 #include "global.h"
 
@@ -56,10 +65,13 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "window.h"
 #include "funcs.h"
 #include "graph.h"
+#include "mdi.h"
+#include "postscript.h"
 
+#include "userpref.h"
+#include "mysql.h"
 #ifdef	HAVE_MOTIF
 #include "io-motif.h"
-#include <Xm/Xm.h>
 #endif
 
 #ifdef HAVE_LIBGTK
@@ -74,26 +86,15 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "panic.h"
 #endif
 
-/* This variable is non-zero if the spreadsheet has been changed in any way */ 
-int modified = 0;
-
-/* User settable options */
-int bkgrnd_recalc = 1;
-int auto_recalc = 1;
-int a0 = 0;
-int topclear = 0;
-
-/* This is how frequently the alarm should go off. */
-unsigned int alarm_seconds = 1;
-
-/* This is whether the alarm should go off at all. */
-unsigned int alarm_active = 1;
-
-/* Jump here on error.  This simply restarts the top 
- * level command loop.  User state should have been 
- * reset appropriately before the longjmp.
+/*
+ * The ultimate global variable
  */
-jmp_buf error_exception;
+#if 1
+struct OleoGlobal *Global = NULL;
+#else
+struct OleoGlobal	__tempGlobal,
+			*Global = &__tempGlobal;
+#endif
 
 /* These are the hooks used to do file-io. */
 void (*read_file) (FILE *, int) = oleo_read_file;
@@ -105,7 +106,7 @@ static char	option_separator = '\t';
 static char	*option_format = NULL;
 int		option_filter = 0;
 
-static char short_options[] = "VqfxthsFS";
+static char short_options[] = "VqfxthsFSv";
 static struct option long_options[] =
 {
 	{"version",		0,	NULL,			'V'},
@@ -118,6 +119,7 @@ static struct option long_options[] =
 	{"space",		0,	NULL,			'S'},
 	{"format",		1,	NULL,			'F'},
 	{"filter",		0,	NULL,			'-'},
+	{"version",		0,	NULL,			'v'},
 	{NULL,			0,	NULL,			0}
 };
 
@@ -166,15 +168,77 @@ got_sig (int sig)
 {
 }
 
+/*
+ * Simple table of variables to set
+ *	All these variables belong in a UserPreferences structure.
+ */
+
+struct UserPreferences UserPreferences;
+
+/*
+ * How to use this ?
+ *
+ * Set "name" to the string we'll find in the .oleorc.
+ * Put a pointer to the variable to assign to in "var".
+ *
+ * If a function needs to be called upon encountering this option,
+ *	put its address in "trigger".
+ * If it's an integer, put the value to assign in "value".
+ * If it's a string, set "copynext" to 1. The string in the option will be
+ *	copied to the variable.
+ *
+ * Set "write" to 1 if this needs to be saved in .oleorc.
+ *	Even then, only saved to file if integer has the "value" from the table,
+ *	or if the string is non-empty.
+ * Set cont to 1 if processing in do_set_option is to continue.
+ */
+static struct pref {
+	char	*name;
+	void	*var;
+	int	value;
+	void	(*trigger)(char *);
+	int	copynext;
+	int	write;
+	int	cont;
+} Preferences [] = {
+	{ "a0",		&UserPreferences.a0,		1,	NULL,	0, 1, 1},
+	{ "noa0",	&UserPreferences.a0,		0,	NULL,	0, 1, 1},
+	{ "bgcolor",	&UserPreferences.bgcolor,	0,	NULL,	1, 0, 1},
+	{ "file",	&UserPreferences.file_type,	0,	NULL,	1, 1, 1},
+	{ "encoding",	&UserPreferences.encoding,	0,	OleoUserPrefEncoding,	1, 1, 0},
+	{ NULL,	0 }
+};
+
 /* An parser for the language grokked by option setting commands. */
 
 static int 
 do_set_option (char *ptr)
 {
-  int set_opt = 1;
+  int	set_opt = 1;
+  int	i, l;
+  char	*p;
 
   while (*ptr == ' ')
     ptr++;
+
+  for (l=0,p=ptr; *p && !isspace(*p); p++) l++;
+ 
+  for (i=0; Preferences[i].name; i++)
+	if (strncmp(ptr, Preferences[i].name, l) == 0) {
+		if (Preferences[i].trigger != NULL)
+			(Preferences[i].trigger)(ptr);
+
+		if (Preferences[i].copynext) {
+			ptr += strlen(Preferences[i].name) + 1;
+			((char *)Preferences[i].var) = strdup(ptr);
+		} else if (Preferences[i].var)
+			*((int *)Preferences[i].var) = Preferences[i].value;
+
+		if (Preferences[i].cont == 0)
+			return;
+		break;
+	}
+
   if (!strincmp ("no", ptr, 2))
     {
       ptr += 2;
@@ -184,17 +248,17 @@ do_set_option (char *ptr)
     }
   if (!stricmp ("auto", ptr))
     {
-      auto_recalc = set_opt;
+      Global->auto_recalc = set_opt;
       return 0;
     }
   if (!stricmp ("bkgrnd", ptr) || !stricmp ("background", ptr))
     {
-      bkgrnd_recalc = set_opt;
+      Global->bkgrnd_recalc = set_opt;
       return 0;
     }
   if (!stricmp ("a0", ptr))
     {
-      a0 = set_opt;
+      Global->a0 = set_opt;
       io_repaint ();
       return 0;
     }
@@ -232,7 +296,7 @@ do_set_option (char *ptr)
 	}
       else if (!stricmp ("sylk", ptr))
 	{
-	  sylk_a0 = 1;
+	  Global->sylk_a0 = 1;
 	  read_file = sylk_read_file;
 	  write_file = sylk_write_file;
 	  set_file_opts = sylk_set_options;
@@ -240,7 +304,7 @@ do_set_option (char *ptr)
 	}
       else if (!stricmp ("sylk-noa0", ptr))
 	{
-	  sylk_a0 = 0;
+	  Global->sylk_a0 = 0;
 	  read_file = sylk_read_file;
 	  write_file = sylk_write_file;
 	  set_file_opts = sylk_set_options;
@@ -287,6 +351,52 @@ do_set_option (char *ptr)
   return 1;
 }
 
+void
+save_preferences(void)
+{
+	char	*home = getenv("HOME");
+	char	*rc, *rc2;
+	FILE	*fp;
+	int	i;
+
+	rc = malloc(strlen(home) + strlen(RCFILE) + 4);
+	rc2 = malloc(strlen(home) + strlen(RCFILE) + 4);
+
+	sprintf(rc, "%s/%s", home, RCFILE);
+	sprintf(rc2, "%s/%s.bak", home, RCFILE);
+
+	(void)unlink(rc2);
+	rename(rc, rc2);
+
+	free(rc2);
+
+	fp = fopen(rc, "w");
+	if (fp == NULL) {
+		io_info_msg("Couldn't save preferences in %s: %s",
+			rc, strerror(errno));
+		free(rc);
+		return;
+	}
+
+	for (i=0; Preferences[i].name; i++)
+		if (Preferences[i].write) {
+			if (Preferences[i].copynext) {
+				if (strlen((char *)Preferences[i].var) != 0)
+				    fprintf(fp, "set-option %s %s\n",
+					Preferences[i].name,
+					(char *) Preferences[i].var);
+			} else if (Preferences[i].value == *(int *)Preferences[i].var)
+				fprintf(fp, "set-option %s\n",
+					Preferences[i].name);
+		}
+
+	fclose(fp);
+
+	io_info_msg("Saved preferences to %s", rc);
+
+	free(rc);
+}
+
 void
 set_options (char * ptr)
 {
@@ -301,11 +411,11 @@ show_options (void)
   int fmts;
   char *data_buf[9];
 
-  n = auto_recalc;
+  n = Global->auto_recalc;
   io_text_start ();
 
   io_text_line ("auto-recalculation: %s        Recalculate in background: %s",
-		n ? " on" : "off", bkgrnd_recalc ? "on" : "off");
+		n ? " on" : "off", Global->bkgrnd_recalc ? "on" : "off");
   io_text_line ("make backup files:  %s        Copy files into backups:   %s",
 	__make_backups ? " on" : "off", __backup_by_copying ? "on" : "off");
 
@@ -447,9 +557,9 @@ void
 write_mp_options (FILE *fp)
 {
   fprintf (fp, "O;%sauto;%sbackground;%sa0;ticks %d\n",
-	   auto_recalc ? "" : "no",
-	   bkgrnd_recalc ? "" : "no",
-	   a0 ? "" : "no",
+	   Global->auto_recalc ? "" : "no",
+	   Global->bkgrnd_recalc ? "" : "no",
+	   Global->a0 ? "" : "no",
 	   cell_timer_seconds);
 }
 
@@ -478,7 +588,7 @@ set_var (struct rng *val, char *var)
 {
   char *ret;
 
-  modified = 1;
+  Global->modified = 1;
   ret = new_var_value (var, strlen(var), val);
 
   if (ret)
@@ -524,7 +634,7 @@ show_var (char *ptr)
       io_error_msg ("There is no '%s'", ptr);
       return;
     }
-  if (a0)
+  if (Global->a0)
     {
       if (v->v_rng.lr != v->v_rng.hr || v->v_rng.lc != v->v_rng.hc)
 	/* FOO */ sprintf (print_buf, "%s $%s$%u:$%s$%u", v->var_name, col_to_str (v->v_rng.lc), v->v_rng.lr, col_to_str (v->v_rng.hc), v->v_rng.hr);
@@ -541,7 +651,7 @@ show_a_var (char *name, struct var *v)
 {
   if (v->var_flags == VAR_UNDEF)
     return;
-  if (a0)
+  if (Global->a0)
     {
       if (v->v_rng.lr != v->v_rng.hr || v->v_rng.lc != v->v_rng.hc)
 	/* FOO */ io_text_line ("%-20s  $%s$%u:$%s$%u", v->var_name, col_to_str (v->v_rng.lc), v->v_rng.lr, col_to_str (v->v_rng.hc), v->v_rng.hr);
@@ -636,7 +746,7 @@ read_variables (FILE * fp)
 			return;	/* actually, io_error_msg never returns. */
 		      }
 		    else
-		      modified = 1;
+		      Global->modified = 1;
 		    break;
 		  }
 		case VAR_RANGE:
@@ -687,6 +797,13 @@ add_usr_cmds (struct cmd_func *new_cmds)
   return num_funcs - 1;
 }
 
+/*
+ * The variable below makes it possible for the program "what" to identify
+ * which version of oleo this executable is.
+ */
+
+static char *what_version = "@(#)" PACKAGE " " VERSION ;
+
 static void
 show_usage (void)
 {
@@ -721,9 +838,133 @@ continue_oleo (int sig)
     cont_curses ();
 }
 
-int display_opened = 0;
+void InitializeGlobals(void)
+{
+  Global->display_opened = 0;
+  Global->return_from_error = 0;
 
-extern int sneaky_linec;
+  /* Initialize stuff that's now in Global */
+  Global->bkgrnd_recalc = 1;
+  Global->auto_recalc = 1;
+  Global->a0 = 0;
+  Global->topclear = 0;
+  Global->alarm_seconds = 1;
+  Global->alarm_active = 1;
+  /* From window.c */
+  scr_lines = 24;
+  scr_cols = 80;
+  user_input = 1;
+  user_status = 2;
+  Global->input = 0;
+  Global->status = 1;
+  input_rows = 1;
+  status_rows = 1;
+  default_right_border = 0;
+  default_bottom_border = 0;
+  nwin = 0;
+  cwin = 0;
+  wins = 0;
+  win_id = 1;
+
+  Global->sylk_a0 = 1;
+
+  Global->user_height_scale = 1.;
+  Global->user_width_scale = 1.;
+  Global->height_scale = 1.;
+  Global->width_scale = 1.;
+
+  Global->cell_font_point_size = 12;
+  Global->block_on_getch = 1;
+
+  Global->display_formula_mode = 0;
+  Global->auto_motion_direction = magic_down;
+  Global->sl_sep = '\t';
+
+  Global->CurrentPrintDriver = &PostScriptPrintDriver;
+  Global->zoom = 1.0;
+
+  Global->mouse_id = 0;
+
+  UserPreferences.run_load_hooks = 1;
+  /* End initialize */
+
+  __make_backups = 1;
+}
+
+void
+oleo_catch_signals(void (*h)(int))
+{
+  /*
+   * These probably don't all need to be ifdef, but
+   * it is harmless.
+   */
+#ifdef SIGCONT
+  signal (SIGCONT, continue_oleo);
+#endif
+#ifdef SIGPIPE
+  signal (SIGPIPE, h);
+#endif
+
+  /*
+   * It makes little sense to block all signals when using X
+   */
+  if ((! using_x) && (! using_gtk) && (! using_motif)) {
+#ifdef SIGINT
+  signal (SIGINT, h);
+#endif
+#ifdef SIGQUIT
+  signal (SIGQUIT, h);
+#endif
+#ifdef SIGILL
+  signal (SIGILL, h);
+#endif
+#ifdef SIGEMT
+  signal (SIGEMT, h);
+#endif
+#ifdef SIGBUS
+  signal (SIGBUS, h);
+#endif
+#ifdef SIGSEGV
+  signal (SIGSEGV, h);
+#endif
+#ifdef	SIGHUP
+  signal(SIGHUP, h);
+#endif
+#ifdef	SIGTRAP
+  signal(SIGTRAP, h);
+#endif
+#ifdef	SIGABRT
+  signal(SIGABRT, h);
+#endif
+#ifdef	SIGFPE
+  signal(SIGFPE, h);
+#endif
+#ifdef	SIGSYS
+  signal(SIGSYS, h);
+#endif
+#ifdef	SIGALRM
+  signal(SIGALRM, h);
+#endif
+#ifdef	SIGTERM
+  signal(SIGTERM, h);
+#endif
+#ifdef	SIGXCPU
+  signal(SIGXCPU, h);
+#endif
+#ifdef	SIGVTALRM
+  signal(SIGVTALRM, h);
+#endif
+#ifdef	SIGPROF
+  signal(SIGPROF, h);
+#endif
+#ifdef	SIGUSR1
+  signal(SIGUSR1, h);
+#endif
+#ifdef	SIGUSR2
+  signal(SIGUSR2, h);
+#endif
+  }
+}
 
 int 
 main (int argc, char **argv)
@@ -740,7 +981,10 @@ main (int argc, char **argv)
   textdomain(PACKAGE);
 #endif
 
-  __make_backups = 1;
+  MdiInitialize();	/* Create initial Global structure */
+  PlotInit();
+  AllocateDatabaseGlobal();
+  InitializeGlobals();
 
   /* Set up the minimal io handler. */
 #if 0
@@ -766,9 +1010,10 @@ main (int argc, char **argv)
 
 	switch (opt)
 	  {
+	  case 'v':
 	  case 'V':
 	    printf(_("%s %s\n"), GNU_PACKAGE, VERSION);
-            printf(_("Copyright (C) 1997-1998 Free Software Foundation, Inc.\n"));
+            printf(_("Copyright  1992-2000 Free Software Foundation, Inc.\n"));
             printf(_("%s comes with ABSOLUTELY NO WARRANTY.\n"), GNU_PACKAGE);
             printf(_("You may redistribute copies of %s\n"), PACKAGE);
             printf(_("under the terms of the GNU General Public License.\n"));
@@ -892,7 +1137,7 @@ main (int argc, char **argv)
 #ifndef X_DISPLAY_MISSING
   if (!no_x) {
     get_x11_args (&argc, argv);
-    if (io_x11_display_name) {
+    if (Global->io_x11_display_name) {
       x11_graphics ();
       using_x = TRUE;
       no_curses = TRUE;
@@ -911,11 +1156,14 @@ main (int argc, char **argv)
   io_open_display ();
 
   init_graphing ();
+  PrintInit();
+  OleoSetEncoding(OLEO_DEFAULT_ENCODING);
 
-  if (setjmp (error_exception))
+  if (setjmp (Global->error_exception))
   {
-	  fprintf (stderr, _("Error in the builtin init scripts (a bug!).\n"));
-	  exit (69);
+	fprintf (stderr, _("Error in the builtin init scripts (a bug!).\n"));
+	io_close_display(69);
+	exit (69);
   }
   else
   {
@@ -924,87 +1172,19 @@ main (int argc, char **argv)
 	  run_init_cmds ();
   }
 
-  /* These probably don't all need to be ifdef, but
-   * it is harmless.
-   */
-#ifdef SIGCONT
-  signal (SIGCONT, continue_oleo);
-#endif
-#ifdef SIGPIPE
-  signal (SIGPIPE, got_sig);
-#endif
-
-  /*
-   * It makes little sense to block all signals when using X
-   */
-  if ((! using_x) && (! using_gtk) && (! using_motif)) {
-#ifdef SIGINT
-  signal (SIGINT, got_sig);
-#endif
-#ifdef SIGQUIT
-  signal (SIGQUIT, got_sig);
-#endif
-#ifdef SIGILL
-  signal (SIGILL, got_sig);
-#endif
-#ifdef SIGEMT
-  signal (SIGEMT, got_sig);
-#endif
-#ifdef SIGBUS
-  signal (SIGBUS, got_sig);
-#endif
-#ifdef SIGSEGV
-  signal (SIGSEGV, got_sig);
-#endif
-#ifdef	SIGHUP
-  signal(SIGHUP, got_sig);
-#endif
-#ifdef	SIGTRAP
-  signal(SIGTRAP, got_sig);
-#endif
-#ifdef	SIGABRT
-  signal(SIGABRT, got_sig);
-#endif
-#ifdef	SIGFPE
-  signal(SIGFPE, got_sig);
-#endif
-#ifdef	SIGSYS
-  signal(SIGSYS, got_sig);
-#endif
-#ifdef	SIGALRM
-  signal(SIGALRM, got_sig);
-#endif
-#ifdef	SIGTERM
-  signal(SIGTERM, got_sig);
-#endif
-#ifdef	SIGXCPU
-  signal(SIGXCPU, got_sig);
-#endif
-#ifdef	SIGVTALRM
-  signal(SIGVTALRM, got_sig);
-#endif
-#ifdef	SIGPROF
-  signal(SIGPROF, got_sig);
-#endif
-#ifdef	SIGUSR1
-  signal(SIGUSR1, got_sig);
-#endif
-#ifdef	SIGUSR2
-  signal(SIGUSR2, got_sig);
-#endif
-  }
+  oleo_catch_signals(&got_sig);
 
   /* Read the init file. */
   {
     volatile int x;
     for (x = 0; x < init_fpc; ++x)
       {
-	if (setjmp (error_exception))
+	if (setjmp (Global->error_exception))
 	  {
 	    fprintf (stderr, _("   error occured in init file %s near line %d."),
-		     init_file_names [x], sneaky_linec);
+		     init_file_names [x], Global->sneaky_linec);
 	    io_info_msg(_("   error occured in init file %s near line %d."),
-		     init_file_names [x], sneaky_linec);
+		     init_file_names [x], Global->sneaky_linec);
 	  }
 	else
 	  if (!ignore_init_file)
@@ -1021,7 +1201,7 @@ main (int argc, char **argv)
       /* fixme: record file name */
 
     if ((fp = fopen (argv[optind], "r"))) {
-	  if (setjmp (error_exception)) {
+	  if (setjmp (Global->error_exception)) {
 	    fprintf (stderr, _(", error occured reading '%s'\n"), argv[optind]);
 	    io_info_msg(_(", error occured reading '%s'\n"), argv[optind]);
 	  } else
@@ -1047,7 +1227,7 @@ main (int argc, char **argv)
 
   io_recenter_cur_win ();
 
-  display_opened = 1;
+  Global->display_opened = 1;
 
 #if 0
   /* FIXME - Find better way of doing this */
@@ -1060,21 +1240,22 @@ main (int argc, char **argv)
 #ifdef	HAVE_MOTIF
   if (using_motif) {
     motif_build_gui();
-    setjmp (error_exception);
+    setjmp (Global->error_exception);
     motif_main_loop();
   } else {
     /* Compile with Motif but don't run with it. */
     while (1)
     {
-      setjmp (error_exception);
+      setjmp (Global->error_exception);
       command_loop (0, 0);
     }
   }
 #else
   while (1)
     {
-      setjmp (error_exception);
+      setjmp (Global->error_exception);
       command_loop (0, 0);
     }
 #endif
+  return (0); /* Never Reached! */
 }
