@@ -1,25 +1,35 @@
-/*      Copyright (C) 1990, 1992, 1993 Free Software Foundation, Inc.
+/*
+ * $Id: io-utils.c,v 1.36 2000/08/10 21:02:50 danny Exp $
+ *
+ * Copyright  1990, 1992, 1993, 2000 Free Software Foundation, Inc.
+ *
+ * This file is part of Oleo, the GNU Spreadsheet.
+ *
+ * Oleo is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * Oleo is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along with
+ * Oleo; see the file COPYING.  If not, write to the Free Software Foundation, 675
+ * Mass Ave, Cambridge, MA 02139, USA.
+ */
 
-   This file is part of Oleo, the GNU Spreadsheet.
-
-   Oleo is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
-   Oleo is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License along with
-   Oleo; see the file COPYING.  If not, write to the Free Software Foundation, 675
-   Mass Ave, Cambridge, MA 02139, USA.  */
+static char *rcsid = "$Id: ";
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include <stdio.h>
 #include <ctype.h>
 #include "sysdef.h"
@@ -33,6 +43,13 @@
 #include "io-term.h"
 #include "cmd.h"
 #include "sylk.h"
+#ifdef	HAVE_MOTIF
+#include "io-motif.h"
+#endif
+#include "basic.h"
+#include "oleofile.h"
+#include "sc.h"
+#include "list.h"
 
 #ifdef	HAVE_TIME_H
 #include <time.h>
@@ -994,7 +1011,7 @@ cell_name (CELLREF rr, CELLREF cc)
 
   num = num ? 0 : 1;
 
-  if (a0)
+  if (Global->a0)
     {
       ptr = &strs[num][9];
       sprintf (ptr, "%u", rr);
@@ -1068,7 +1085,7 @@ range_name (struct rng *rng)
   if ((lr == hr) && (lc == hc)) {
       sprintf (ptr, "%s", cell_name (lr, lc));
   } else {
-      if (a0)
+      if (Global->a0)
         sprintf (ptr, "%s:%s", cell_name (lr, lc), cell_name (hr, hc));
       else {
           if (lr == hr && lc != hc)
@@ -1327,10 +1344,10 @@ write_file_generic_2(FILE *fp, struct rng *rng, char *format)
 	if (!stricmp ("oleo", format)) {
 		oleo_write_file(fp, rng);
 	} else if (!stricmp ("sylk", format)) {
-		sylk_a0 = 1;
+		Global->sylk_a0 = 1;
 		sylk_write_file(fp, rng);
 	} else if (!stricmp ("sylk-noa0", format)) {
-		sylk_a0 = 0;
+		Global->sylk_a0 = 0;
 		sylk_write_file(fp, rng);
 	} else if (!stricmp ("sc", format)) {
 		sc_write_file(fp, rng);
@@ -1373,10 +1390,10 @@ read_file_generic_2(FILE *fp, int ismerge, char *format, char *name)
 	if (stricmp ("oleo", format) == 0) {
 		oleo_read_file(fp, ismerge);
 	} else if (stricmp ("sylk", format) == 0 || stricmp ("slk", format) == 0) {
-		sylk_a0 = 0;	/* FIX ME */
+		Global->sylk_a0 = 0;	/* FIX ME */
 		sylk_read_file(fp, ismerge);
 	} else if (stricmp ("sylk-noa0", format) == 0) {
-		sylk_a0 = 0;
+		Global->sylk_a0 = 0;
 		sylk_read_file(fp, ismerge);
 	} else if (stricmp ("sc", format) == 0) {
 		sc_read_file(fp, ismerge);
@@ -1390,7 +1407,7 @@ read_file_generic_2(FILE *fp, int ismerge, char *format, char *name)
 		list_set_separator(',');
 		list_read_file(fp, ismerge);
 	} else if (stricmp("dbf", format) == 0) {
-#ifdef HAVE_LIBXBASE
+#if defined(HAVE_LIBXBASE) || defined(HAVE_LIBXDB)
 		ReadXbaseFile(name, ismerge);
 #else
 		io_error_msg ("Cannot read XBASE file (xbase not compiled into " PACKAGE ")");
@@ -1461,7 +1478,7 @@ read_file_generic(FILE *fp, int ismerge, char *format, char *name)
 			oleo_read_file(fp, ismerge);
 	}
 
-	recalculate();
+	recalculate(1);
 }
 
 void
@@ -1474,32 +1491,30 @@ fmt_get_format()
 {
 }
 
-static char	*current_file = NULL;
-
 void FileSetCurrentFileName(const char *s)
 {
-	if (current_file)
-		free(current_file);
-	current_file = NULL;
+	if (Global->FileName)
+		free(Global->FileName);
+	Global->FileName = NULL;
 	if (s)
-		current_file = strdup(s);
+		Global->FileName = strdup(s);
 
 #if HAVE_MOTIF
 	if (using_motif)
-		MotifSetWindowName(current_file);
+		MotifSetWindowName(Global->FileName);
 #endif
 }
 
 char *FileGetCurrentFileName(void)
 {
-	return current_file;
+	return Global->FileName;
 }
 
 void FileCloseCurrentFile(void)
 {
-	if (current_file)
-		free(current_file);
-	current_file = NULL;
+	if (Global->FileName)
+		free(Global->FileName);
+	Global->FileName = NULL;
 
 #if HAVE_MOTIF
 	if (using_motif)
@@ -1507,3 +1522,24 @@ void FileCloseCurrentFile(void)
 #endif
 }
 
+void OleoSetEncoding(char *s)
+{
+	if (Global && Global->encoding)
+		free(Global->encoding);
+	Global->encoding = strdup(s);
+
+	PrintSetEncoding(s);
+}
+
+char *OleoGetEncoding(void)
+{
+	return Global->encoding;
+}
+
+void OleoUserPrefEncoding(char *s)
+{
+	char	*p = s + 9;	/* Get past "encoding" */
+
+	for (; *p && isspace(*p); p++) ;
+	OleoSetEncoding(p);
+}
