@@ -1,24 +1,33 @@
-/*	Copyright (C) 1992, 1993 Free Software Foundation, Inc.
-	
-	This program is free software; you can redistribute it and/or modify
-	it under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 2, or (at your option)
-	any later version.
-	
-	This program is distributed in the hope that it will be useful,
-	but WITHOUT ANY WARRANTY; without even the implied warranty of
-	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-	GNU General Public License for more details.
-	
-	You should have received a copy of the GNU General Public License
-	along with this software; see the file COPYING.  If not, write to
-	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
+#undef	I18N_VERBOSE
+#undef	X_I18N
+/*
+ * $Id: io-x11.c,v 1.25 2000/08/10 21:02:50 danny Exp $
+ *
+ *	Copyright  1992, 1993, 1999 Free Software Foundation, Inc.
+ * 	
+ * 	This program is free software; you can redistribute it and/or modify
+ * 	it under the terms of the GNU General Public License as published by
+ * 	the Free Software Foundation; either version 2, or (at your option)
+ * 	any later version.
+ * 	
+ * 	This program is distributed in the hope that it will be useful,
+ * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * 	GNU General Public License for more details.
+ * 	
+ * 	You should have received a copy of the GNU General Public License
+ * 	along with this software; see the file COPYING.  If not, write to
+ * 	the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #ifndef	HAVE_MOTIF
 #ifndef X_DISPLAY_MISSING
 
@@ -48,7 +57,8 @@
 #include "regions.h"
 #include "input.h"
 #include "info.h"
-
+
+#include <locale.h>
 
 #if defined(HAVE_RINT)
 extern double rint (double);
@@ -61,7 +71,6 @@ extern char * x_get_string_resource (XrmDatabase, char *, char *);
 extern XrmDatabase x_load_resources (Display *, char *, char *);
 extern char * getenv (const char *);
 
-
 static char *emergency_font_name = "8x13";
 static char *cell_font_name = "times_roman12";
 static char *default_font_name = "8x13";
@@ -69,13 +78,9 @@ static char *input_font_name = "8x13";
 static char *status_font_name = "6x10";
 static char *text_line_font_name = "8x13";
 static char *label_font_name = "5x8";
-int cell_font_point_size = 12;
 static char *default_bg_color_name = "black";
 static char *default_fg_color_name = "white";
 
-/* If non-0 and !no_x, these display functions will be used. */
-char *io_x11_display_name = 0;
-
 /* The geometry of the first window. */
 static int geom_x = 0;
 static int geom_y = 0;
@@ -85,6 +90,11 @@ static char geom_string[] = "675x350+0+0";
 
 /* This global is used only during command line and .Xdefaults handling. */
 static Display * theDisplay;
+static struct sXport *thePort;
+
+/* i18n */
+static XIC	xic = 0;
+static XIM	xim = 0;
 
 static char * rdb_class_name = "Oleo";
 static char * rdb_name = "oleo";
@@ -156,17 +166,18 @@ get_x11_args (int * argc_p, char ** argv)
   
   /* Compute the display from either resources or getenv. */
   
-  io_x11_display_name =
+  Global->io_x11_display_name =
     x_get_string_resource (argv_resources, class_of ("Display"),
 			   name_of ("display"));
 #endif					       
-  if (!io_x11_display_name)
-    io_x11_display_name = ck_savestr (getenv ("DISPLAY"));
+  if (!Global->io_x11_display_name)
+    Global->io_x11_display_name = ck_savestr (getenv ("DISPLAY"));
   
-  if (!io_x11_display_name)
+  if (!Global->io_x11_display_name)
     return;
   
-  theDisplay = XOpenDisplay (io_x11_display_name);
+  setlocale(LC_ALL, "");	/* i18n */
+  theDisplay = XOpenDisplay (Global->io_x11_display_name);
   if (!theDisplay)
     panic ("Can not connect to X.  Check your DISPLAY evironment variable.");
   
@@ -205,9 +216,7 @@ get_x11_args (int * argc_p, char ** argv)
   }
 }
 
-
 static int x11_opened = 0;
-static struct sXport *thePort;
 typedef struct sXport *Xport;
 
 struct sXport
@@ -247,6 +256,198 @@ struct sXport
   struct input_view input_view;
 };
 
+#if defined(XlibSpecificationRelease) && (XlibSpecificationRelease >= 6)
+static void
+GetXIC(Display *theDisplay)
+{
+	XIMStyles	*xim_styles;
+	XIMStyle	input_style = 0;
+	char		*p, *buf, *b, *k;
+	int		i, j, found = 0;
+#if 0
+	XVaNestedList	list;
+	XFontSet	fontset;
+	char		**missing_charsets;
+	int		num_missing_charsets = 0;
+	char		*default_string;
+	long		mask;
+#endif
+
+	static char	*styles[] =
+	{
+		"OverTheSpot",
+		"OffTheSpot",
+		"Root",
+		"Root",
+		NULL
+	};
+	static char *preeditTypes = "OffTheSpot,OverTheSpot,Root";
+
+	static XIMStyle style_bits[] =
+	{
+	/* OverTheSpot */	XIMPreeditPosition | XIMStatusArea,
+	/* OffTheSpot */	XIMPreeditArea | XIMStatusArea,
+	/* Root */		XIMPreeditNothing | XIMStatusNothing,
+	/* Not really root */	XIMPreeditNone | XIMStatusNone
+	};
+
+	if (!XSupportsLocale()) {
+#ifdef	I18N_VERBOSE
+		fprintf(stderr, "X doesn't support this locale\n");
+#endif
+	}
+
+	if (xim == NULL && (p = XSetLocaleModifiers("@im=none")) != NULL) {
+		xim = XOpenIM(theDisplay, NULL, NULL, NULL);
+#ifdef	I18N_VERBOSE
+		if (xim) fprintf(stderr, "XOpenIM with @im=none succeeded\n");
+#endif
+	}
+	if (xim == NULL && (p = XSetLocaleModifiers("")) != NULL) {
+		xim = XOpenIM(theDisplay, NULL, NULL, NULL);
+#ifdef	I18N_VERBOSE
+		if (xim) fprintf(stderr, "XOpenIM with '' succeeded\n");
+#endif
+	}
+	if (!xim) {
+#ifdef	I18N_VERBOSE
+		fprintf(stderr, "Failed to open input method\n");
+#endif
+		return;
+	}
+
+	if (XGetIMValues(xim, XNQueryInputStyle, &xim_styles, NULL) || xim_styles == NULL) {
+#ifdef	I18N_VERBOSE
+		fprintf(stderr, "Input method doesn't support any style\n");
+#endif
+		XCloseIM(xim);
+		return;
+	}
+
+	/* Match input styles between X server + input method and ourselves */
+	for (i = 0, found = False; styles[i] && !found; i++) {
+#ifdef	I18N_VERBOSE
+		fprintf(stderr, "Trying %s (0x%X)\n", styles[i], style_bits[i]);
+#endif
+
+		if (strstr(preeditTypes, styles[i]) == 0) {
+			continue;
+		}
+		for (j = 0; j < xim_styles->count_styles; j++) {
+			if (style_bits[i] == xim_styles->supported_styles[j]) {
+				found = True;
+				input_style = style_bits[i];
+				break;
+			}
+		}
+	}
+
+#if 0
+	fontset = XCreateFontSet(theDisplay,
+		"-adobe-helvetica-*-r-*-*-*-120-*-*-*-*-*-*,\
+		-misc-fixed-*-r-*-*-*-130-*-*-*-*-*-*",
+		&missing_charsets, &num_missing_charsets, &default_string);
+
+	if (num_missing_charsets > 0) {
+		fprintf(stderr, PACKAGE " : the %d missing charsets\n", num_missing_charsets);
+		for (i=0; i<num_missing_charsets; i++)
+			fprintf(stderr, "\t%d - %s\n", i, missing_charsets[i]);
+	}
+
+	list = XVaCreateNestedList(0,XNFontSet,fontset,NULL);
+#endif
+
+	xic = XCreateIC(xim,
+			XNInputStyle,		input_style,
+			XNClientWindow,		thePort->window,
+			XNFocusWindow,		thePort->window,
+#if 0
+			XNPreeditAttributes,	list,
+			XNStatusAttributes,	list,
+#endif
+		NULL);
+
+#if 0
+	XFree(list);
+#endif
+
+#ifdef	I18N_VERBOSE
+	if (xic)
+		fprintf(stderr, "We have an IC\n");
+	else
+		fprintf(stderr, "No IC\n");
+#endif
+
+#if 0
+	/* UGLY HACK !!! */
+
+	XGetICValues(xic, XNFilterEvents, &mask, NULL);
+#ifdef	I18N_VERBOSE
+	fprintf(stderr, "Mask is 0x%X\n", mask);
+#endif
+
+	XSelectInput(theDisplay, thePort->window,
+		ExposureMask | KeyPressMask | StructureNotifyMask | mask);
+
+	XSetICFocus(xic);
+#endif
+}
+
+static void
+xi18nGetFocus()
+{
+	if (xic) {
+#ifdef	I18N_VERBOSE
+		fprintf(stderr, "Get Focus\n");
+#endif
+		XSetICFocus(xic);
+	}
+}
+
+static void
+xi18nLoseFocus()
+{
+	if (xic) {
+#ifdef	I18N_VERBOSE
+		fprintf(stderr, "Lose Focus\n");
+#endif
+		XUnsetICFocus(xic);
+	}
+}
+
+static int
+xi18nlookup(XEvent *evp, char *buf, int nbytes, KeySym *keysym, Status *status)
+{
+	int	i;
+	static Status	s;
+
+#ifdef	X_I18N
+	if (xic) {
+		i = XmbLookupString(xic, evp, buf, nbytes, keysym, &s);
+	} else {
+		i = XLookupString(evp, buf, nbytes, keysym, &s);
+	}
+
+#ifdef	I18N_VERBOSE
+	fprintf(stderr, "xi18nlookup (%d bytes): %d %c\t", i,
+			buf[0], isprint(buf[0]) ? buf[0] : '.');
+	fprintf(stderr, "status %s",
+		(s == XLookupKeySym) ? "XLookupKeySym" : 
+		(s == XLookupNone) ? "XLookupNone" : 
+		(s == XLookupBoth) ? "XLookupBoth" : 
+		(s == XBufferOverflow) ? "XBufferOverflow" : 
+		(s == XLookupChars) ? "XLookupChars" : "");
+	fprintf(stderr, "\n");
+#endif
+
+#else
+	i = XLookupString(evp, buf, nbytes, keysym, &s);
+#endif
+	if (status)	*status = s;
+	return i;
+}
+#endif
+
 static void
 beep (Xport xport)
 {
@@ -261,7 +462,6 @@ xio_bell (void)
 {
   beep (thePort);
 }
-
 
 static void
 xdraw_text_item (Xport xport, int c, int r, int wid, int hgt,
@@ -341,7 +541,7 @@ xio_redraw_input_cursor (int on)
   int start;
   XTextItem cursor_text;
   int cwid;
-  int ypos = (iv->current_info ? 0 : input);
+  int ypos = (iv->current_info ? 0 : Global->input);
   char * inp;
 
   inp = (iv->input_area
@@ -393,7 +593,7 @@ xio_redraw_input (void)
 {
   {
     struct input_view * iv = &thePort->input_view;
-    int ypos = (iv->current_info ? 0 : input);
+    int ypos = (iv->current_info ? 0 : Global->input);
 
     if (iv->redraw_needed == NO_REDRAW)
       return;
@@ -521,15 +721,11 @@ io_col_to_input_pos (int c)
     return iv->visibility_begin + cpos - 1;
   }
 }
-
 
 /****************************************************************
  * Low level Input
  ****************************************************************/
 
-/* To be like curses, we offer this option: */
-static int block_on_getch = 1;
-
 /* This is the buffer of decoded keyboard events. */
 static char *input_buf = 0;
 static int input_buf_allocated = 0;
@@ -539,7 +735,7 @@ static int se_buf_allocated = 0;
 static int se_chars_buffered = 0;
 #define MAX_KEY_TRANSLATION	(1024)
 
-static XComposeStatus compose;
+static Status compose;
 
 static void
 xio_scan_for_input (int blockp)
@@ -556,8 +752,8 @@ xio_scan_for_input (int blockp)
     {
       if (resize_pending && !events_pending)
 	{
-	  if (   ((pendingh / height_scale) > 9.)
-	      && ((pendingw / width_scale) > 9.))
+	  if (   ((pendingh / Global->height_scale) > 9.)
+	      && ((pendingw / Global->width_scale) > 9.))
 	      {
 		io_set_scr_size (pendingh, pendingw);
 		resize_pending = 0;
@@ -580,6 +776,14 @@ xio_scan_for_input (int blockp)
 
 	      switch (event_return.type)
                 {
+		case FocusIn:
+			xi18nGetFocus();
+			break;
+
+		case FocusOut:
+			xi18nLoseFocus();
+			break;
+
 	          case KeyPress:
                     /* Put keypress send_events aside until a <Return>. */
 	            if (se_chars_buffered + MAX_KEY_TRANSLATION 
@@ -592,9 +796,14 @@ xio_scan_for_input (int blockp)
 	                se_buf =
 		          (char *) ck_remalloc (se_buf, se_buf_allocated);
 	              }
+#if 0
 	            len = XLookupString (&event_return,
 			           se_buf + se_chars_buffered,
 			           MAX_KEY_TRANSLATION, &se_keysym, &compose);
+#else
+		    len = xi18nlookup(&event_return, se_buf + se_chars_buffered,
+				MAX_KEY_TRANSLATION, &se_keysym, &compose);
+#endif
 
 	            if ((len == 1) && (event_return.xkey.state & Mod1Mask))
 	              se_buf[se_chars_buffered] |= 0x80;
@@ -603,23 +812,23 @@ xio_scan_for_input (int blockp)
                       {
 			jmp_buf tmp_exception;
 
-			bcopy(error_exception, tmp_exception, sizeof(jmp_buf));
+			bcopy(Global->error_exception, tmp_exception, sizeof(jmp_buf));
 		        se_buf[se_chars_buffered] = '\0';
 
-			if (setjmp (error_exception))
+			if (setjmp (Global->error_exception))
 			  {
 			    fprintf(stderr, "Error in send_event command: %s\n", se_buf);
 			    se_buf[0] = '\0';
 			    se_chars_buffered = 0;
 			    set_info (0);
 		            pop_unfinished_command ();
-			    bcopy(tmp_exception, error_exception, sizeof(jmp_buf));
-			    longjmp (error_exception, 1); /**/
+			    bcopy(tmp_exception, Global->error_exception, sizeof(jmp_buf));
+			    longjmp (Global->error_exception, 1); /**/
 			  }
 			else
 			  {
 			    execute_command (se_buf);
-			    bcopy(tmp_exception, error_exception, sizeof(jmp_buf));
+			    bcopy(tmp_exception, Global->error_exception, sizeof(jmp_buf));
 			  }
 		        se_buf[0] = '\0';
 		        se_chars_buffered = 0;
@@ -637,6 +846,14 @@ xio_scan_for_input (int blockp)
       
       switch (event_return.type)
 	{
+		case FocusIn:
+			xi18nGetFocus();
+			break;
+
+		case FocusOut:
+			xi18nLoseFocus();
+			break;
+
 	case ClientMessage:
 	  if (event_return.xclient.data.l[0] == thePort->wm_delete_window)
 	    {
@@ -657,14 +874,18 @@ xio_scan_for_input (int blockp)
 		(char *) ck_remalloc (input_buf, input_buf_allocated);
 	    }
 	}
-      
-      
+ 
       switch (event_return.type)
 	{
 	case KeyPress:
+#if 0
 	  len = XLookupString (&event_return,
 			       input_buf + chars_buffered,
 			       MAX_KEY_TRANSLATION, 0, &compose);
+#else
+	  len = xi18nlookup(&event_return,input_buf + chars_buffered,
+				MAX_KEY_TRANSLATION, 0, &compose);
+#endif
 	  if ((len == 1) && (event_return.xkey.state & Mod1Mask))
 	    input_buf[chars_buffered] |= 0x80;
 	  chars_buffered += len;
@@ -704,6 +925,14 @@ xio_scan_for_input (int blockp)
 	  resize_pending = 1;
 	  break;
 	  
+	case FocusIn:
+		xi18nGetFocus();
+		break;
+
+	case FocusOut:
+		xi18nLoseFocus();
+		break;
+
 	default:
 	  break;
 	}
@@ -711,7 +940,6 @@ xio_scan_for_input (int blockp)
     }
   while (events_pending || !chars_buffered);
 }
-
 
 static int
 xio_input_avail (void)
@@ -740,7 +968,7 @@ xio_read_kbd (char *buffer, int size)
 static void
 xio_nodelay (int delayp)
 {
-  block_on_getch = delayp;
+  Global->block_on_getch = delayp;
 }
 
 static int
@@ -749,7 +977,7 @@ xio_getch (void)
   char buf;
   
   if (!chars_buffered)
-    io_scan_for_input (block_on_getch);
+    io_scan_for_input (Global->block_on_getch);
   
   if (chars_buffered)
     {
@@ -759,7 +987,6 @@ xio_getch (void)
   else
     return 0;
 }
-
 
 /****************************************************************
  * Low level support for input and status areas.
@@ -788,8 +1015,6 @@ cram (int cols, XFontStruct *font, char *text, int len, char *continuation)
 }
 
 
-
-
 static void
 xset_text (XTextItem *xtext, char *text, int len)
 {
@@ -804,15 +1029,8 @@ xset_text (XTextItem *xtext, char *text, int len)
     xtext->chars[len++] = ' ';
 }
 
-
-
 /* The input area. */
 
-
-
-
-
-
 /*
  * Low level interface to the input area specificly.
  */
@@ -829,11 +1047,11 @@ draw_input (void)
   if (!input_text.chars)
     {
       XFillRectangle (thePort->dpy, thePort->window, thePort->neutral_gc,
-		      input, 0, scr_cols, input_rows);
+		      Global->input, 0, scr_cols, input_rows);
       return;
     }
   input_text.font = thePort->input_font->fid;
-  xdraw_text_item (thePort, 0, input, scr_cols, input_rows, thePort->input_font,
+  xdraw_text_item (thePort, 0, Global->input, scr_cols, input_rows, thePort->input_font,
 		  thePort->input_gc, &input_text, 1);
   if (input_more_mode)
     {
@@ -843,7 +1061,7 @@ draw_input (void)
       more_text.nchars = 6;
       more_text.delta = 0;
       more_text.font = thePort->input_font->fid;
-      xdraw_text_item (thePort, scr_cols - mwid, input, mwid, input_rows,
+      xdraw_text_item (thePort, scr_cols - mwid, Global->input, mwid, input_rows,
 		      thePort->input_font, thePort->standout_input_gc,
 		      &more_text, 1);
     }
@@ -856,39 +1074,38 @@ draw_input (void)
       cursor_text.chars = input_text.chars + input_cursor;
       cursor_text.nchars = 1;
       cursor_text.font = thePort->input_font->fid;
-      xdraw_text_item (thePort, start, input, cwid, input_rows,
+      xdraw_text_item (thePort, start, Global->input, cwid, input_rows,
 		      thePort->input_font, thePort->standout_input_gc,
 		      &cursor_text, 1);
     }
 }
-
 
 static void
 xio_clear_input_before (void)
 {
   textout = 0;
-  if (topclear == 2)
+  if (Global->topclear == 2)
     {
       int x;
       for (x = 0; x < input_text.nchars; ++x)
 	input_text.chars[x] = ' ';
       input_cursor = 0;
       draw_input ();
-      topclear = 0;
+      Global->topclear = 0;
     }
 }
 
 static void
 xio_clear_input_after (void)
 {
-  if (topclear)
+  if (Global->topclear)
     {
       int x;
       for (x = 0; x < input_text.nchars; ++x)
 	input_text.chars[x] = ' ';
       input_cursor = 0;
       draw_input ();
-      topclear = 0;
+      Global->topclear = 0;
     }
 }
 
@@ -905,7 +1122,6 @@ set_input (char *text, int len, int cursor)
   input_cursor = cursor;
   draw_input ();
 }
-
 
 /*
  * Low level interface to the status area specificly.
@@ -918,7 +1134,7 @@ xdraw_status (void)
   if (!x11_opened || thePort->input_view.current_info)
     return;
   if (user_status)
-    xdraw_text_item (thePort, 0, status, scr_cols,
+    xdraw_text_item (thePort, 0, Global->status, scr_cols,
 		    status_rows, thePort->status_font,
 		    thePort->status_gc, &status_text, 1);
 }
@@ -930,7 +1146,6 @@ xset_status (char *text)
   status_text.font = thePort->status_font->fid;
   xdraw_status ();
 }
-
 
 /****************************************************************
  * High level interfaces for the input and status areas.
@@ -982,7 +1197,7 @@ xio_update_status (void)
       dec = 0;
       dlen = 0;
     }
-  
+
   ptr = cell_value_string (curow, cucol, 1);
   plen = XTextWidth (thePort->status_font, ptr, strlen (ptr));
   
@@ -1047,18 +1262,16 @@ xio_update_status (void)
     free (assembled);
   }
 }
-
 
 static int
 xio_get_chr (char *prompt)
 {
   int len = strlen (prompt);
   set_input (prompt, len, len);
-  topclear = 2;
+  Global->topclear = 2;
   draw_input ();
   return io_getch ();
 }
-
 
 /*
  * Multi-line informational messages to the user:
@@ -1076,7 +1289,6 @@ static int text_damaged = 0;
 static struct text_line *text_lines = 0;
 extern int auto_recalc;
 
-
 /****************************************************************
  * Low level support for the cell windows.
  ****************************************************************/
@@ -1115,7 +1327,6 @@ static XRectangle *cliprectv = 0;
 
 #define GC_CACHE_SIZE	10
 static struct cell_gc *cell_gc_cache = 0;
-
 
 /* This takes the full name of an X11 font, and returns its point size (in
  * tenths of a point.  If the string is not a valid font name, 0 is returned.
@@ -1156,7 +1367,7 @@ cell_gc (Xport port, struct font_memo *font_passed, int cursor)
   char *font_name = font->names->x_name;
   double scale = font->scale;
   struct cell_gc *c = cell_gc_cache;
-  if (cell_gc_basis != cell_font_point_size)
+  if (cell_gc_basis != Global->cell_font_point_size)
     {
       do
 	{
@@ -1171,7 +1382,7 @@ cell_gc (Xport port, struct font_memo *font_passed, int cursor)
 	  c = c->next;
 	}
       while (c != cell_gc_cache);
-      cell_gc_basis = cell_font_point_size;
+      cell_gc_basis = Global->cell_font_point_size;
     }
   else
     {
@@ -1213,7 +1424,7 @@ cell_gc (Xport port, struct font_memo *font_passed, int cursor)
     if (fontv)
       {
 	int x, best;
-	int ideal_size = rint (scale * (double) cell_font_point_size * 10.);
+	int ideal_size = rint (scale * (double) Global->cell_font_point_size * 10.);
 	int best_dist;
 	best = 0;
 	best_dist = ideal_size - name_to_ps (fontv[0]);
@@ -1275,8 +1486,6 @@ cell_gc (Xport port, struct font_memo *font_passed, int cursor)
   return c;
 }
 
-
-
 /* This is the data for an oleo window, displayed under X. */
 
 enum kinds_of_layout_needed
@@ -1294,8 +1503,6 @@ struct x_window
   int label_damage;
   Xport port;
 };
-
-
 
 static void
 collect_clipping (xx_IntRectangle rect)
@@ -1334,7 +1541,6 @@ clip_to_intrectangle (struct x_window * xwin, xx_IntRectangle rect)
   collect_clipping (rect);
   ++clipcnt;
 }
-
 
 static void 
 place_text (xx_IntRectangle out, struct display *disp, struct cell_display *cd, XFontStruct *font, char *string)
@@ -1365,8 +1571,6 @@ place_text (xx_IntRectangle out, struct display *disp, struct cell_display *cd,
     }
   xx_IRinit (out, xout, yout, wout, hout);
 }
-
-
 
 static void
 x_metric (struct cell_display *cd, struct display *disp)
@@ -1381,8 +1585,6 @@ x_metric (struct cell_display *cd, struct display *disp)
       place_text (&cd->goal, disp, cd, cgc->font, cd->unclipped);
     }
 }
-
-
 
 static struct x_window *
 x_win (Xport port, struct window *win, int rebuild)
@@ -1426,9 +1628,6 @@ flush_x_windows (Xport port)
     }
 }
 
-
-
-
 static void
 record_damage (Xport port, int x, int y, int w, int h)
 {
@@ -1466,11 +1665,9 @@ record_damage (Xport port, int x, int y, int w, int h)
       xwin = xwin->next;
     }
 
-  if (((input + input_rows) >= y) && (input <= y + h))
+  if (((Global->input + input_rows) >= y) && (Global->input <= y + h))
     port->input_view.redraw_needed = FULL_REDRAW;
 }
-
-
 
 static void
 xio_pr_cell_win (struct window *win, CELLREF r, CELLREF c, CELL *cp)
@@ -1485,23 +1682,27 @@ xio_pr_cell_win (struct window *win, CELLREF r, CELLREF c, CELL *cp)
 	xwin->layout_needed = damaged_display;
     }
 }
-
 
 static void
 xio_repaint_win (struct window *win)
 {
+  if (! Global->display_opened)
+	return;
+
   x_win (thePort, win, 1);
   XFillRectangle (thePort->dpy, thePort->window, thePort->neutral_gc,
 		  win->win_over, win->win_down, win->numc, win->numr);
   record_damage (thePort, win->win_over, win->win_down, win->numc, win->numr);
 }
-
-
 
 static void
 xio_repaint (void)
 {
   struct window *win;
+
+  if (! Global->display_opened)
+	return;
+
   if (thePort->input_view.current_info)
     {
       thePort->input_view.info_redraw_needed = 1;
@@ -1511,7 +1712,6 @@ xio_repaint (void)
   for (win = wins; win < &wins[nwin]; win++)
     xio_repaint_win (win);
 }
-
 
 static void
 draw_cell (struct x_window *xwin, struct cell_display *cd_passed, int cursor)
@@ -1594,7 +1794,6 @@ draw_cell (struct x_window *xwin, struct cell_display *cd_passed, int cursor)
     }
   }
 }
-
 
 /* Cell values */
 
@@ -1660,7 +1859,6 @@ xio_display_cell_cursor (void)
       set_cursor (1);
     }
 }
-
 
 static int xx[10] = {1, 1, 1, 1, 1, 1, 1, 1, 1, 1};
 
@@ -1752,7 +1950,7 @@ draw_labels (void)
 	      int hgt = get_scaled_height (cr);
 	      if (hgt)
 		{
-		  if (a0)
+		  if (Global->a0)
 		    sprintf (buf, "%d", cr);
 		  else
 		    sprintf (buf, "R%d", cr);
@@ -1782,7 +1980,7 @@ draw_labels (void)
 		{
 		  int txtwid;
 		  char *ptr;
-		  if (a0)
+		  if (Global->a0)
 		    ptr = col_to_str (cr);
 		  else
 		    {
@@ -1817,7 +2015,6 @@ draw_labels (void)
 	xwin->label_damage = 0;
       }
 }
-
 
 /* Refresh the existing image. */
 
@@ -1955,7 +2152,6 @@ xio_redisp (void)
 	}
     }
 }
-
 
 static XFontStruct *
 reasonable_font (Xport port, char *name)
@@ -2049,6 +2245,10 @@ xio_open_display (void)
 		   &thePort->wm_delete_window, 1);
 #endif
   
+#if defined(XlibSpecificationRelease) && (XlibSpecificationRelease >= 6)
+  GetXIC(theDisplay);
+#endif
+
   gcv.foreground = thePort->bg_color_pixel;
   gcv.background = thePort->bg_color_pixel;
   thePort->neutral_gc =
@@ -2157,7 +2357,7 @@ xio_open_display (void)
   }
   
   XSelectInput (thePort->dpy, thePort->window,
-		(ExposureMask | StructureNotifyMask | KeyPressMask
+		(ExposureMask | StructureNotifyMask | KeyPressMask | FocusChangeMask
 		 | ButtonPressMask));
   
   io_init_windows (geom_h, geom_w, 1, 2,
@@ -2170,8 +2370,8 @@ xio_open_display (void)
   
   {
     struct cell_gc *cgc = cell_gc (thePort, default_font(), 0);
-    height_scale = cgc->font->ascent + cgc->font->descent;
-    width_scale = XTextWidth (cgc->font, "M", 1);
+    Global->height_scale = cgc->font->ascent + cgc->font->descent;
+    Global->width_scale = XTextWidth (cgc->font, "M", 1);
   }
 
   /* Setup the arrow keys.  Modifiers effect the last character
@@ -2259,7 +2459,7 @@ xio_open_display (void)
 
 
 extern void
-xio_close_display (void)
+xio_close_display(int e)
 {
   XCloseDisplay (thePort->dpy);
   x11_opened = 0;
@@ -2272,8 +2472,6 @@ xio_flush (void)
 }
 
 
-
-
 void
 xio_command_loop (int i)
 {
@@ -2324,11 +2522,11 @@ set_x_default_point_size (int l)
   
   if (l > 4)
     {
-      cell_font_point_size = l;
+      Global->cell_font_point_size = l;
       {
 	struct cell_gc *cgc = cell_gc (thePort, default_font(), 0);
-	height_scale = cgc->font->ascent + cgc->font->descent;
-	width_scale = cgc->font->max_bounds.width;
+	Global->height_scale = cgc->font->ascent + cgc->font->descent;
+	Global->width_scale = cgc->font->max_bounds.width;
       }
       io_set_scr_size (scr_lines, scr_cols);
     }
