@@ -1,5 +1,7 @@
 /*
- * Copyright (C) 1990-1999 Free Software Foundation, Inc.
+ * $Id: oleofile.c,v 1.24 2000/11/22 19:33:01 danny Exp $
+ *
+ * Copyright  1990-2000 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +22,10 @@
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "funcdef.h"
 #include <stdio.h>
 #include <ctype.h>
@@ -42,6 +48,8 @@
 
 #include "graph.h"
 
+#include "oleosql.h"
+
 /* These functions read and write OLEO style files. */
 
 void
@@ -72,9 +80,11 @@ oleo_read_file (fp, ismerge)
   int old_a0;
   int next_a0;
 
-  old_a0 = a0;
+  Global->return_from_error = 1;
+
+  old_a0 = Global->a0;
   next_a0 = old_a0;
-  a0 = 0;
+  Global->a0 = 0;
   lineno = 0;
   if (!ismerge)
     clear_spreadsheet ();
@@ -96,8 +106,7 @@ oleo_read_file (fp, ismerge)
 	  break;
 	case '%':		/* Font or pixel size data. */
 	  ptr++;
-	  switch (*ptr)
-	    {
+	  switch (*ptr) {
 	    case 'F':		/* %F font-name */
 	      if (fnt_map_size == fnt_map_alloc)
 		{
@@ -130,7 +139,7 @@ oleo_read_file (fp, ismerge)
 		}
 		break;
 	      }
-	    default:
+	    default:		/* % with something invalid */
 	      goto bad_field;
 	    }
 	  break;
@@ -143,8 +152,7 @@ oleo_read_file (fp, ismerge)
 	      if (*ptr != ';')
 		goto bad_field;
 	      ptr++;
-	      switch (*ptr++)
-		{
+	      switch (*ptr++) {
 		  int clo, chi, cwid;
 		case 'C':	/* Column from rows 1 to 255 */
 		  czcol = astol (&ptr);
@@ -308,8 +316,9 @@ oleo_read_file (fp, ismerge)
 		  clo = astol (&ptr);
 		  chi = astol (&ptr);
 		  cwid = astol (&ptr) + 1;
-		  for (; clo <= chi; clo++)
-		    set_width (clo, cwid);
+		  for (; clo <= chi; clo++) {
+			set_width (clo, cwid);
+		    }
 		  break;
 
 		case 'H':	/* JF: extension */
@@ -546,13 +555,13 @@ oleo_read_file (fp, ismerge)
 	  break;
 	  /* JF extension: read uset-settable options */
 	case 'O':
-	  a0 = next_a0;
+	  Global->a0 = next_a0;
 	  read_mp_options (ptr + 2);
-	  next_a0 = a0;
-	  a0 = 0;
+	  next_a0 = Global->a0;
+	  Global->a0 = 0;
 	  break;
 	case 'G':	/* Graph data */
-/*	  fprintf(stderr, "Graph input line '%s'\n", cbuf);	/* */
+/*	  fprintf(stderr, "Graph input line '%s'\n", cbuf);	*/
 	  switch (*(ptr+1)) {
 	  case 'T':	/* Graph Title */
 	    graph_set_title(cbuf+2);
@@ -560,6 +569,21 @@ oleo_read_file (fp, ismerge)
 	  case 'D':	/* Axis title : GDxtitle */
 	    graph_set_axis_title(cbuf[2], cbuf+3);
 	    break;
+	  case 't':	/* Graph data title : Gtxtitle */
+	    graph_set_data_title(cbuf[2] - '0', cbuf+3);
+	    break;
+	  case 'a':	/* Automatic axis setting : Gax0 or Gax1 */
+	    graph_set_axis_auto(cbuf[2] - '0', cbuf[3] == '1');
+	    break;
+	  case 'o':	/* Whether to draw line to offscreen data points */
+	    graph_set_linetooffscreen(cbuf[2] == '1');
+	    break;
+	  case 'r':	/* Axis range : GrxlVALUE , l = 0 for low, 1 for high */
+	    if (cbuf[3] == '0')
+		graph_set_axis_lo('x' + cbuf[2] - '0', &cbuf[4]);
+	    else if (cbuf[3] == '1')
+		graph_set_axis_hi('x' + cbuf[2] - '0', &cbuf[4]);
+	    break;
 	  case 'L':	/* Axis logness GLx0 or GLx1 */
 	    graph_set_logness(cbuf[2], 1, cbuf[3] == '1');
 	    break;
@@ -574,33 +598,64 @@ oleo_read_file (fp, ismerge)
 	      r.lr = b;
 	      r.hc = c;
 	      r.hr = d;
-	      graph_set_data(i, &r, 'h', 'r');
+	      graph_set_data(i, &r);
 	    }
 	    break;
+	  case 'm':	/* Tick marks Gmxts, x = 0 or 1, t = 0 .. 4 (the tick type),
+			 * s is the format string, if any.
+			 */
+	    {
+		int	axis, tp;
+
+		axis = cbuf[2] - '0';
+		tp = cbuf[3] - '0';
+
+		graph_set_axis_ticks(axis, tp, strdup(&cbuf[4]));
+	    }
+		break;
 	  default:
 	    fprintf(stderr, "Graph: invalid line '%s'\n", cbuf);
 	  }
 	  break;
+	case 'D':	/* Database Access */
+		ptr++;
+		switch (*ptr) {
+		case 'u':
+			DatabaseSetUser(ptr+1);
+			break;
+		case 'h':
+			DatabaseSetHost(ptr+1);
+			break;
+		case 'n':
+			DatabaseSetName(ptr+1);
+			break;
+		default:
+			io_error_msg("Line %d - unknown code %s\n", lineno, cbuf);
+		}
+		break;
 	default:
 	bad_field:
-	  a0 = old_a0;
+	  Global->a0 = old_a0;
 	  if (!ismerge)
 	    clear_spreadsheet ();
 	  io_recenter_all_win ();
 	  io_error_msg ("Line %d: Unknown OLEO line \"%s\"", lineno, cbuf);
+	  Global->return_from_error = 0;
 	  return;
-	}
+	}	/* End of switch */
     }
-  if (!feof (fp))
-    {
-      if (!ismerge)
-	clear_spreadsheet ();
-      io_recenter_all_win ();
-      io_error_msg ("read-file: read-error near line %d.", lineno);
-      return;
+  if (!feof (fp)) {
+	if (!ismerge)
+		clear_spreadsheet ();
+	io_recenter_all_win ();
+	io_error_msg ("read-file: read-error near line %d.", lineno);
+	Global->return_from_error = 0;
+	return;
     }
-  a0 = next_a0;
+  Global->a0 = next_a0;
   io_recenter_all_win ();
+
+  Global->return_from_error = 0;
 }
 
 static char *
@@ -752,7 +807,7 @@ oleo_write_file (fp, rng)
   /* All versions of the oleo file format should have a 
    * version cookie on the second line.
    */
-  (void) fprintf (fp, "# format 2.0 (requires Oleo 1.6.14 or higher)\n");
+  (void) fprintf (fp, "# format 2.1 (requires Oleo 1.99.9 or higher)\n");
 
   /* If no range given, write the entire file */
   if (!rng)
@@ -795,8 +850,8 @@ oleo_write_file (fp, rng)
       write_mp_options (fp);
     }
 
-  old_a0 = a0;
-  a0 = 0;
+  old_a0 = Global->a0;
+  Global->a0 = 0;
 
   find_widths (rng->lc, rng->hc);
   w = next_width (&c);
@@ -854,10 +909,16 @@ oleo_write_file (fp, rng)
 	      if (cp->cell_font->id_memo < 0)
 		{
 		  cp->cell_font->id_memo = fnt_map_size++;
-		  fprintf (fp, "%%F%s,%s,%f\n",
+		  if (isnan(cp->cell_font->scale))
+			fprintf (fp, "%%F%s,%s,%f\n",
 			   cp->cell_font->names->x_name,
 			   cp->cell_font->names->ps_name,
 			   cp->cell_font->scale);
+		  else
+			fprintf (fp, "%%F%s,%s,%f\n",
+			   cp->cell_font->names->x_name,
+			   cp->cell_font->names->ps_name,
+			   1.0);
 		}
 	    }
 	  (void) fprintf (fp, "F;");
@@ -976,10 +1037,47 @@ oleo_write_file (fp, rng)
     fprintf(fp, "GDy%s\n", s);
   }
 
+  for (i=0; i<NUM_DATASETS; i++) {
+	if (graph_get_data_title(i))
+		fprintf(fp, "Gt%c%s\n", i + '0', graph_get_data_title(i));
+  }
+
+  /* Axis range : GrxlVALUE , l = 0 for low, 1 for high */
+  for (i=0; i< 2; i++) {
+	double	d;
+	d = graph_get_axis_lo(i);
+	if (! isnan(d))
+		fprintf(fp, "Gr%c0%f\n", '0' + i, d);
+	d = graph_get_axis_hi(i);
+	if (! isnan(d))
+		fprintf(fp, "Gr%c1%f\n", '0' + i, d);
+  }
+
+  /* Automatic axis setting : Gax0 or Gax1 */
+  fprintf(fp, "Ga0%c\n", graph_get_axis_auto(0) ? '1' : '0');	/* X axis */
+  fprintf(fp, "Ga1%c\n", graph_get_axis_auto(1) ? '1' : '0');	/* Y axis */
+
+  /* Draw line to offscreen data points */
+  fprintf(fp, "Go%c\n", graph_get_linetooffscreen() ? '1' : '0');
+
+  /* Axis tick marks */
+  fprintf(fp, "Gm0%c%s\n",
+	'0' + graph_get_axis_ticktype(0),
+	graph_get_axis_tickformat(0));
+  fprintf(fp, "Gm1%c%s\n",
+	'0' + graph_get_axis_ticktype(1),
+	graph_get_axis_tickformat(1));
+
+  /* Database stuff */
+  if (DatabaseInitialised()) {
+	fprintf(fp, "Dn%s\n", DatabaseGetName() ? DatabaseGetName() : "");
+	fprintf(fp, "Dh%s\n", DatabaseGetHost() ? DatabaseGetHost() : "");
+	fprintf(fp, "Du%s\n", DatabaseGetUser() ? DatabaseGetUser() : "");
+  }
 
   /* End of writing */
   (void) fprintf (fp, "E\n");
-  a0 = old_a0;
+  Global->a0 = old_a0;
 }
 
 int
