@@ -1,25 +1,32 @@
-/*	Copyright (C) 1990, 1992, 1993 Free Software Foundation, Inc.
-
-This file is part of Oleo, the GNU Spreadsheet.
-
-Oleo is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-Oleo is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Oleo; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
+/*
+ * $Id: ref.c,v 1.13 2000/08/10 21:02:51 danny Exp $
+ *
+ * Copyright  1990, 1992, 1993 Free Software Foundation, Inc.
+ * 
+ * This file is part of Oleo, the GNU Spreadsheet.
+ * 
+ * Oleo is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ * 
+ * Oleo is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Oleo; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "funcdef.h"
 #include <stdio.h>
 #include <ctype.h>
@@ -157,6 +164,26 @@ new_value (CELLREF row, CELLREF col, char *string)
   return 0;
 }
 
+char *
+quote_new_value(CELLREF row, CELLREF col, char *string)
+{
+	int	l;
+	char	*r, *s;
+
+	l = strlen(string);
+
+	s = (char *)malloc(l+3);
+	s[0] = '"';
+	s[1] = '\0';
+	strcat(s, string);
+	strcat(s, "\"");
+
+	r = new_value(row, col, s);
+	free(s);
+
+	return r;
+}
+
 /* This sets the cell to a constant, stored in VALUE, whose type is in TYPE */
 char *
 set_new_value (CELLREF row, CELLREF col, int type, union vals *value)
