@@ -1,25 +1,33 @@
-/*	Copyright (C) 1990, 1992, 1993 Free Software Foundation, Inc.)
-
-This file is part of Oleo, the GNU Spreadsheet.
-
-Oleo is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-Oleo is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Oleo; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+/*
+ * $Id: regions.c,v 1.16 2000/08/10 21:02:51 danny Exp $
+ *
+ * Copyright  1990, 1992, 1993 Free Software Foundation, Inc.
+ *
+ * This file is part of Oleo, the GNU Spreadsheet.
+ *
+ * Oleo is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ *
+ * Oleo is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with Oleo; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include "funcdef.h"
 #include <stdio.h>
 #include "sysdef.h"
@@ -37,6 +45,7 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "sort.h"
 #include "basic.h"
 
+/* Apparently "all_rng" is read-only, therefore need not be put in Global. */
 struct rng all_rng = {MIN_ROW, MIN_COL, MAX_ROW, MAX_COL};
 
 /* Take a struct rng (R) and init its elements to R1 C1 R2 C2, making sure
@@ -74,7 +83,7 @@ delete_region (struct rng *where)
   CELLREF rr, cc;
   CELL *pp;
 
-  modified = 1;
+  Global->modified = 1;
 
   find_cells_in_range (where);
   while ((pp = next_row_col_in_range (&rr, &cc)))
@@ -104,7 +113,7 @@ lock_region (struct rng *where, int locked)
 {
   CELL *cp;
 
-  modified = 1;
+  Global->modified = 1;
   make_cells_in_range (where);
   while ((cp = next_cell_in_range ()))
     SET_LCK (cp, locked);
@@ -116,7 +125,7 @@ format_region (struct rng *where, int fmt, int just)
   CELL *cp;
   CELLREF rr, cc;
 
-  modified = 1;
+  Global->modified = 1;
   make_cells_in_range (where);
   while ((cp = next_row_col_in_range (&rr, &cc)))
     {
@@ -135,7 +144,7 @@ precision_region (struct rng *where, int precision)
   CELL *cp;
   CELLREF rr, cc;
 
-  modified = 1;
+  Global->modified = 1;
   make_cells_in_range (where);
   while ((cp = next_row_col_in_range (&rr, &cc)))
     {
@@ -344,7 +353,7 @@ set_to_region (struct rng *fm, struct rng *to)
   if (((fm->lr <= to->lr && to->lr <= fm->hr) || (fm->lr <= to->hr && to->hr <= fm->hr))
       && ((fm->lc <= to->lc && to->lc <= fm->hc) || (fm->lc <= to->hc && to->hc <= fm->hc)))
     return -1;
-  modified = 1;
+  Global->modified = 1;
   return ret;
 }
 
@@ -672,11 +681,10 @@ move_region (struct rng *fm, struct rng *to)
  *
  * --FB 1997.12.17
  */
-  if (mkrow != NON_ROW)
-    {
+  if (mkrow != NON_ROW) {
       mkrow = to->lr + abs(to->lr - to->hr);
       mkcol = to->lc + abs(to->lc - to->hc);
-    }
+  }
   goto_region (to);
   return;
 }
@@ -701,11 +709,10 @@ copy_region (struct rng *fm, struct rng *to)
       if (rf == fm->hr)
 	rf = fm->lr - 1;
     }
-  if (mkrow != NON_ROW)
-    {
+  if (mkrow != NON_ROW) {
       mkrow = to->lr + abs(to->lr - to->hr);
       mkcol = to->lc + abs(to->lc - to->hc);
-    }
+  }
   goto_region (to);
 }
 
@@ -770,7 +777,7 @@ sort_region (void)
       io_error_msg ("Can't sort this region!");
       return;
     }
-  modified = 1;
+  Global->modified = 1;
   if (scdiff != ecdiff)
     {
       erdiff = 0;
