@@ -1,27 +1,37 @@
-/*	Copyright (C) 1990, 1992, 1993 Free Software Foundation, Inc.
+/*
+ * $Id : $
+ *
+ * Copyright  1990, 1992, 1993 Free Software Foundation, Inc.
+ * 
+ * This file is part of Oleo, the GNU Spreadsheet.
+ * 
+ * Oleo is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ * 
+ * Oleo is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with Oleo; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
-This file is part of Oleo, the GNU Spreadsheet.
-
-Oleo is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-Oleo is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with Oleo; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include <stdio.h>
 #include <errno.h>
+#include <stdlib.h>
+
 #include "sysdef.h"
 #include <stdarg.h>
 #include <fcntl.h>
@@ -55,15 +65,8 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #define _IOSTRG 0
 #endif
 
-#ifndef _DEBUG_MALLOC_INC
-extern void abort (void);
-extern void *malloc (size_t);
-extern void *calloc (size_t, size_t);
-extern void *realloc (void *, size_t);
-#endif /* _DEBUG_MALLOC_INC */
-
 extern int sys_nerr;
-
+
 struct id
   {
     int flag;
@@ -660,7 +663,7 @@ char *
 err_msg (void)
 {
   int n;
-  static char buf[80];
+  static char buf[80];	/* Static to be able to return its address */
   char *p;
 
   n = errno;
@@ -824,12 +827,11 @@ char_to_string (int ch)
   if (ch == '\\')
     return "\\\\";
 
-  if (ch >= ' ' && ch <= '~')
-    {
+  if (isprint(ch)) {
       buf[3] = ch;
       buf[4] = 0;
       return &buf[3];
-    }
+  }
 
   if (ch & 0x80)
     {
@@ -902,7 +904,7 @@ astol (char **ptr)
     s++;
 
   /* Read in the digits */
-  for (; c = *s; s++)
+  for (; (c = *s); s++)
     {
       if (!isdigit (c) || i > 214748364 || (i == 214748364 && c > (sign > 0 ? '7' : '8')))
 	break;
