@@ -1,23 +1,31 @@
-/*	Copyright (C) 1992, 1993 Free Software Foundation, Inc.
-
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2, or (at your option)
-any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this software; see the file COPYING.  If not, write to
-the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
+/*
+ * $Id: window.c,v 1.9 2000/08/10 21:02:51 danny Exp $
+ *
+ * Copyright  1992, 1993, 1999 Free Software Foundation, Inc.
+ * 
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2, or (at your option)
+ * any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this software; see the file COPYING.  If not, write to
+ * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
+#ifdef	WITH_DMALLOC
+#include <dmalloc.h>
+#endif
+
 #include <stdio.h>
 #include "global.h"
 #include "window.h"
@@ -29,48 +37,6 @@ the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include "lists.h"
 #include "regions.h"
 
-int scr_lines = 24;
-int scr_cols = 80;
-#define LINES scr_lines
-#define COLS scr_cols
-
-
-/* These control the layout of input and status lines. */
-int user_input = 1;		/* As specified (+/- [0-2]). */
-int user_status = 2;
-int input = 0;			/* As row address. */
-int status = 1;
-int input_rows = 1;		/* Size of input and status areas. */
-int status_rows = 1;
-
-/* These control the layout of edge labels. */
-int label_rows;
-int label_emcols;
-
-/* Temporary hacks for displaying multi-line messages. */
-/* If this is non-0, it should be displayed instead of 
- * the windows of cells.  This is a temporary hack.
- * Use set_info to change this.
- */
-struct info_buffer * current_info;
-
-int info_rows;		/* The height of one row of info in a */
-			/* multi-line message. */
-int info_line;		/* In the current info buffer, the first vis. line */
-int info_over;		/* Info scrolls left/right as well as up/down */
-
-
-/* Window borders: */
-int default_right_border = 0;
-int default_bottom_border = 0;
-
-/* The window list. */
-int nwin = 0;
-struct window *cwin = 0;
-struct window *wins = 0;
-static int win_id = 1;
-
-
 /* Low level window operators. */
 
 #define MIN_WIN_HEIGHT(W) (W->bottom_edge_r \
@@ -726,8 +692,8 @@ io_set_input_status (int inp, int stat, int redraw)
 	}
       user_input = new_ui;
       user_status = new_us;
-      input = new_inp;
-      status = new_stat;
+      Global->input = new_inp;
+      Global->status = new_stat;
     }
 }
 
@@ -1073,7 +1039,7 @@ io_write_window_config (struct line * out)
   sprint_line (out, "O;status %d\n", user_status);
   if (nwin > 1)
     {
-      /* ... *//* fixme ? */
+      /* ... */ /* fixme ? */
     }
   for (n = 0; n < nwin; n++)
     {
@@ -1261,20 +1227,14 @@ io_read_window_config (char * line)
       set_options (opts);
     }
 }
-
-
-
-static struct mouse_event *current_mouse;
-static struct mouse_event *free_mouse;
-static int mouse_id = 0;
 
 static void 
 init_mouse (void)
 {
-  current_mouse = free_mouse =
+  Global->current_mouse = Global->free_mouse =
   (struct mouse_event *) ck_malloc (sizeof (struct mouse_event));
-  free_mouse->next = free_mouse;
-  free_mouse->prev = free_mouse;
+  Global->free_mouse->next = Global->free_mouse;
+  Global->free_mouse->prev = Global->free_mouse;
 }
 
 static int mouse_location ();
@@ -1282,19 +1242,19 @@ static int mouse_location ();
 int 
 enqueue_mouse_event (int r, int c, int button, int downp)
 {
-  struct mouse_event *m = free_mouse;
-  if (m->next == current_mouse)
+  struct mouse_event *m = Global->free_mouse;
+  if (m->next == Global->current_mouse)
     {
       m->next =
 	(struct mouse_event *) ck_malloc (sizeof (struct mouse_event));
       m->next->prev = m;
-      m->next->next = current_mouse;
-      current_mouse->prev = m->next;
-      m->seq = mouse_id++;
+      m->next->next = Global->current_mouse;
+      Global->current_mouse->prev = m->next;
+      m->seq = Global->mouse_id++;
       if (m->seq > 255)
 	panic ("Too many mouse events enqueued.");
     }
-  free_mouse = m->next;
+  Global->free_mouse = m->next;
   m->row = r;
   m->col = c;
   m->button = button;
@@ -1306,18 +1266,18 @@ enqueue_mouse_event (int r, int c, int button, int downp)
 void 
 dequeue_mouse_event (struct mouse_event *out, int seq)
 {
-  free_mouse->seq = seq;
-  while (current_mouse->seq != seq)
-    current_mouse = current_mouse->next;
-  if (current_mouse == free_mouse)
+  Global->free_mouse->seq = seq;
+  while (Global->current_mouse->seq != seq)
+    Global->current_mouse = Global->current_mouse->next;
+  if (Global->current_mouse == Global->free_mouse)
     {
       out->seq = seq;
       out->button = MOUSE_QERROR;
       return;
     }
-  *out = *current_mouse;
+  *out = *Global->current_mouse;
   out->next = out->prev = 0;
-  current_mouse = current_mouse->next;
+  Global->current_mouse = Global->current_mouse->next;
 }
 
 
@@ -1359,10 +1319,10 @@ static int
 mouse_location (CELLREF *cr, CELLREF *cc, struct mouse_event *ev)
 {
   int n;
-  if (ev->row >= input && ev->row <= input + input_rows)
+  if (ev->row >= Global->input && ev->row <= Global->input + input_rows)
     return MOUSE_ON_INPUT;
-  if (user_status && ev->row >= status
-      && ev->row <= status + status_rows)
+  if (user_status && ev->row >= Global->status
+      && ev->row <= Global->status + status_rows)
     return MOUSE_ON_STATUS;
   for (n = 0; n < nwin; ++n)
     {
